#!/bin/bash
#
# COPYRIGHT INFORMATION - DO NOT REMOVE
#
# This file is part of the MagicSpam (TM) Project and is
# Copyright (c) LinuxMagic Inc. 2015 All Rights Reserved
#
# This file contains Original Code as created by LinuxMagic Inc.
#
# The Original Code is distributed on an 'AS IS' basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
# ENJOYMENT OR NON-INFRINGEMENT.
#
# Do NOT download, distribute, use or alter this software or file in any
# way without express written permission from LinuxMagic Inc. or its parent
# company Wizard Tower TechnoServices signed by an authorized company officer.
#
# Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
#
# $Id$
#
export PATH=/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/usr/local/sbin

if [ ! -x "/etc/init.d/magicspam-daemon" ]; then
    >&2 echo "Missing init script: is MagicSpam installed?"
    exit 3
fi

if [ ! -x "/etc/init.d/magicspam-milter" ]; then
    >&2 echo "Missing init script: is MagicSpam installed?"
    exit 3
fi
SERVICE=$(which service 2>/dev/null)
SYSTEMCTL=$(which systemctl 2>/dev/null)

####
# wrapper function to 'start' a service by name
#
# @param    string  the service name to start
#
# NOTE: this function is intended to act as an intermediate layer
#       of abstraction between systemd, 'service' (upstart), and classic sysv-init
#
function start_service {
    SVC=$1
    if [ -z "$SVC" ]; then
        return 0
    fi
    # precedence 1: systemctl
    if [ ! -z "$SYSTEMCTL" ] && [ -x "$SYSTEMCTL" ]; then
        $SYSTEMCTL start $SVC
        return $?
    fi
    # precedence 2: upstart
    if [ ! -z "$SERVICE" ] && [ -x "$SERVICE" ]; then
        $SERVICE $SVC start
        return $?
    fi
    # precedence 3: sysv-init
    if [ -x "/etc/init.d/$SVC" ]; then
        /etc/init.d/$SVC start
        return $?
    fi
    echo "Unable to find method to start requested service $SVC"
    return 1
}


/etc/init.d/magicspam-daemon status >/dev/null 2>&1
if [ $? -ne 0 ]; then
    start_service magicspam-daemon
    if [ $? -ne 0 ]; then
        >&2 echo "Failed to start MagicSpam Processing daemon"
        exit 1
    fi
fi

/etc/init.d/magicspam-milter status >/dev/null 2>&1
if [ $? -ne 0 ]; then
    start_service magicspam-milter
    if [ $? -ne 0 ]; then
        >&2 echo "Failed to start MagicSpam Postfix Milter"
        exit 1
    fi
fi

exit 0
