<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2013-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id$
 */

// ensure we have the 'include' paths pointing back to our tree.
define('BASE_DIR', '/usr/share/magicspam/php/');
set_include_path(BASE_DIR . "includes" . PATH_SEPARATOR . get_include_path());
set_include_path(BASE_DIR . "lib" . PATH_SEPARATOR . get_include_path());

// our base requirements
require_once('config.php');         // defines and suite specific utility path information
require_once('functions.php');
require_once('lib.licensemanager.php'); // for accessing the license manager.
require_once('lib.splash_screen.php');

// set the default PHP timezone if necessary
handleTimezone();

// load up the MagicSpam locales, default to English
$locale = getControl("ms_locale", "en-US");
require_once(loadMSLocale());

// and start our session for persistent storage
session_start();

//-----------------------------
// set up our global variables

// flag to distinguished between running on admin or user interface
$isFrontend = FALSE;

// array of error messages stored through each module
$errors = array();

// array of informational messages each module may wish to report
$notices = array();

// flag indicative if we have been called in an ajax only manner
$output_only = false;

// our CSS rendering data - seeded withour global CSS common to all modules
$css = array('magicspam.css', 'msalone.css', 'pro.css', 'splash.css');

// our JS rendering data - seeded with our common scripts
$js = array('magicspam.js', 'css_browser_selector.js', 'pro.js');

// our array of all modules that belong in this system.
$modules = array();

// the administration email address to send any errors to.
$admin_email = getControl('admin_email', '');

// grab the singleton for the license manager binary handle
$license = LicenseManager::getInstance();

// get our posted base level arguments
// NOTE: 'ms_action' instead of 'action' to avoid collision with MVC
//   framework handling that may assume a special meaning of 'action'.
$action = isset($_REQUEST['ms_action']) ? $_REQUEST['ms_action'] : NULL;

// special post handler for AJAX requests
if (isset($_REQUEST["output_only"]) && is_numeric($_REQUEST["output_only"]) && $_REQUEST["output_only"] == 1 ) {
    $output_only = TRUE;
}

// load the modules.
foreach ($pages as $page) {
    loadModule($page);
}

// determine if we are authenticated or not.
$authenticated = loginModule::isAuthenticated();
$module = NULL;

if ($authenticated) {
    // set the 'module' string based on request / defaults
    $module = isset($_REQUEST['module']) ? $_REQUEST['module'] : 'dashboard';
}

// grab the active module.
$activeModule = isset($modules[$module]) ? $modules[$module] : NULL;

// process any actions - setting any feedback that we might need.
if (!is_null($action)) {
    if ( (!$authenticated && is_null($module)) ||
         ($authenticated && $activeModule) ) {
         if (is_null($module)) {
             $module = 'login';
             $activeModule = isset($modules[$module]) ? $modules[$module] : NULL;
         }
         if ($activeModule) {
            $activeModule->actionProcessor($action, $errors, $notices);
         }
    }
}

$authenticated = loginModule::isAuthenticated();
if (!$authenticated) {
    $module = 'login';
} else {
    // redirect to dashboard from login page
    if ($module == 'login') {
        $module = 'dashboard';
    }   

    // retrieve the view for the first time feature announcement if necessary
    $splash = new SplashScreen();
    $splash->setTitle("New MagicSpam Features");
    $splash->setViewFile(CONTROLDIR . ".features_view");
    $splash->setCookie("MS_SPLASH_FEATURES", MAGICSPAM_VERSION, 365);
    $splash_html = $splash->getView();
}

$serviceok = TRUE;
if (isset($modules['serviceStatus'])) {
    $serviceok = $modules['serviceStatus']->isRunning();
}

// now we prep for rendering...
// start by determining what the end effective module is
// if this is a licensed only version...
if (!isFreemium() && $authenticated) {
    // and there is no valid license entered yet...
    // NOTE: this covers: NEW, CORRUPT, EXPIRED
    if (!$license->isValid() && $module != 'logout') {
        $module = 'settings';
        // IF the end user requested something other than system settings..
        if ( (isset($_REQUEST['module']) && $_REQUEST['module'] !== 'settings')
          || (isset($_POST['page']) && $_POST['page'] !== 'system_settings') ) {
            // there are 3 reasons a license is invalid: empty, expired, or corrupt.
            if (!$license->isExpired()) {
                // fall through for 'new' or 'corrupt' - expired cases handled
                // further down as a 'global' message.
                $errors[] = intl('empty_license');
            }
        }
        // XXX: we shouldn't have to play with POST like this... but...
        $_POST['page'] = 'system_settings';
    }
}

// global to all.. if we have a license and it is expired...
if ($license->isExpired() && $authenticated && $module != 'logout') {
    // then tell the world about it
    $errors[] = intl('expired_license');
    // and ALWAYS force to the system options page.
    $module = 'settings';
    $_POST['page'] = 'system_settings';
} else {
    if ($authenticated) {
        // if we have a valid license or are in freemium mode
        if ($license->isValid() || isFreemium()) {
            // then we check that an email address is set.
            if ($admin_email === '' && $module != 'logout') {
                $module = 'settings';
                // IF the end user requested something other than system settings...
                if ( (isset($_REQUEST['module']) && $_REQUEST['module'] !== 'settings')
                  || (isset($_POST['page']) && $_POST['page'] !== 'system_settings') ) {
                    $errors[] = intl('ae_please_complete');
                }
                // XXX: we shouldn't have to play with POST like this... but...
                $_POST['page'] = 'system_settings';
            } else if (!$serviceok) {
                $module = 'settings';
                $_POST['page'] = 'system_settings';
            }
        }
    }
}

// grab the active module.
$activeModule = isset($modules[$module]) ? $modules[$module] : NULL;

if (!empty($errors) && !empty($notices)) {
    $notices = array();
}

if (!$output_only) {
    // render the header.
    renderHeader(
        array_unique(array_merge($css, $activeModule
            ? $activeModule->getCSS() : array())),
        array_unique(array_merge($js, $activeModule
         ? $activeModule->getJS() : array())),
        $errors,
        $notices,
        $activeModule ? $activeModule->getPageTitle() : 'MagicSpam'
    );
}


// and the individual module
if ($activeModule) {
    if (!$output_only) { // not output_only, use the renderModule function
        $activeModule->renderModule();
        // TODO: we may want a way to include i18n even for ajax calls
        $activeModule->renderI18nJS();
        if ($module !== NULL) {
            echo '<!-- end module ' . htmlspecialchars($module) . ' -->' . "\n";
        }
    } else if ($output_only == 1) { // output_only, use the ajax equivalent function.
        $activeModule->renderOutput($modules);
    }

    // render the first time feature announcement
    if (isset($splash_html)) {
        echo $splash_html;
    }
}

if (!$output_only) {
    // and of course the copyright footer
    renderFooter();
}

