/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2011-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: alerts.js 4617 2014-12-17 18:51:59Z shaun $
 */

//! function to show / hide the full message content
/*!
 *  @param  string   contains the id of the element
 *
 *  @return none
 *
 *  @author Robert Bisewski <robertb@linuxmagic.com>
 */
function showMore(element_id) {

    // variables used to store the elements
    var msg;
    var ctnElement;
    var ctnElements;
    var shortMsg;
    var fullMsg;
    var showButtonRow;
    var showMore;

    // Validation: checks if the param exists
    if (!element_id) {
        return;
    }

    // Attempts to grab the element container
    msg = document.getElementById(element_id);
    if (!msg) {
        return;
    }

    // Grabs the notification-container of the msg
    ctnElement  = MS.getElementsByClassName(msg, "notification-container");
    if (!ctnElement) {
        return;
    }

    // Grabs the child nodes of notification-container
    ctnElements = ctnElement[0].childNodes;

    // Validation: checks if the container holds the elements
    if (!ctnElements) {
        return;
    }

    // Cycles thru the child elements, to grab the ids we need
    for (var i = 0; i < ctnElements.length; i++) {
       if (ctnElements[i].id == "shortMessage") {
           shortMsg = ctnElements[i];
       } else if (ctnElements[i].id == "fullMessage") {
           fullMsg = ctnElements[i];
       } else if (ctnElements[i].className == "buttonRow markReadButton") {
           showButtonRow = ctnElements[i].childNodes;
       }
    }

    // Validation: If the contents are non-existant, then ends here
    if (!shortMsg || !fullMsg || !showButtonRow) {
        return;
    }

    // Searchs for the "Read More..." link
    for (var i = 0; i < showButtonRow.length; i++) {
       if (showButtonRow[i].id == "showMore") {
           showMore = showButtonRow[i];
       }
    }

    // Validation: if the show more link is non-existant, ends here...
    if (!showMore) {
        return;
    }

    // Attempts to change the classes of the elements
    if (shortMsg.className === "notification-description") {
        shortMsg.className = "notification-description-hidden";
        fullMsg.className  = "notification-description";
        showMore.innerHTML = MS.intl('sa_read_less');
    } else if (fullMsg.className === "notification-description") {
        shortMsg.className = "notification-description";
        fullMsg.className  = "notification-description-hidden";
        showMore.innerHTML = MS.intl('sa_read_more');
    }
}

//! function to show / hide the read / unread messages
/*!
 *  @param  string  the id of the element we are interesting in
 *
 *  @return none
 *
 *  Author(s): Robert Bisewski <robertb@linuxmagic.com>
 */
function showAlerts(id) {

    // Variable declaration
    var alertBox;
    var unreadAlertList;
    var alertList;
    var hiddenAlertList;
    var noAlerts;
    var showReadUnread;

    // Basic id validation
    if (!id) {
        return;
    }

    // Grabs and checks if the alert containing element is valid
    alertBox = document.getElementById(id);
    if (!alertBox) {
        return;
    }
    // Grabs all of the read / hidden alerts present in the alert container
    unreadAlertList = MS.getElementsByClassName(alertBox, "noteBubble");
    alertList       = MS.getElementsByClassName(alertBox, "noteBubbleRead");
    hiddenAlertList = MS.getElementsByClassName(alertBox, "noteBubbleHidden");
    if (!unreadAlertList || !alertList || !hiddenAlertList) {
        return;
    }

    // Grabs the "Show Read" / "Show Unread" link
    showReadUnread = document.getElementById(id + "_link");
    if (!showReadUnread) {
        return;
    }

    // Grabs the "no alerts / announcements" div
    noAlerts = document.getElementById(id + "_noneMsg");
    if (!noAlerts) {
        return;
    }
    // Cycles thru each of the alerts and shows / hides them
    // Depending on whether the messages are currently being hidden / shown...
    if (showReadUnread.innerHTML.trim() === MS.intl('sa_show_unread')) {

        // Hides the read messages
        for (var i = alertList.length - 1; i >= 0; --i) {
            alertList[i].className = "noteBubbleHidden";
        }
        showReadUnread.innerHTML = MS.intl('sa_show_read');

        // Displays "No System Alerts" if none are present, else hides it
        if (unreadAlertList.length > 0) {
            noAlerts.className = "noAlertsInvisible";
        } else if (unreadAlertList.length < 1) {
            noAlerts.className = "noAlerts";
        }

    } else if (showReadUnread.innerHTML.trim() === MS.intl('sa_show_read')) {
        // Shows the read messages
        for (var i = hiddenAlertList.length - 1; i >= 0; --i) {
            hiddenAlertList[i].className = "noteBubbleRead";
        }
        showReadUnread.innerHTML = MS.intl('sa_show_unread');

        // Unlike other browsers, IE8 HTML-element-arrays are not persistant.
        // Ergo, to make it work properly, we have to regenerate it here.
        alertList = MS.getElementsByClassName(alertBox, "noteBubbleRead");
        
        // Displays "No Announcements" if none are present, else hides it
        if (unreadAlertList.length + alertList.length > 0) {
            noAlerts.className = "noAlertsInvisible";
        } else if (unreadAlertList.length + alertList.length < 1) {
            noAlerts.className = "noAlerts";
        }
    }
}

//! utility function designed for event bubbling
/*!
 *  @param  event   current event
 *  @param  string  the id of the element we are interesting in
 *  @param  string  the image path
 *
 *  @return none
 *
 *  Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 */
function doFormSubmitTrap(evt, id, newpath)
{
    // Variable to store the new image path
    var img_path = "";

    // Event validation
    if (!evt) {
        evt = window.event;
    }

    if (!evt) {
        return;
    }
    evt.cancelBubble = true;
    if (evt.stopPropagation) evt.stopPropagation();

    // Element validation
    if (!id) {
        return;
    }

    
    // Attempts to show / hide the messages
    // if a new image path is specifid, and its a valid length string,
    // go ahead and set the image to that location
    if (newpath && newpath.length > 0) {
        img_path = newpath;
    }
    showAlerts(id);
    
    // convert the 'tabid' to an object
    var tab = document.getElementById(id);
    if (!tab) {
        return;
    }	

    var classes = tab.className;
    var n = classes.search('Active');
    var images = document.getElementsByTagName('img');
 
    if (n == -1) {

        tab.className = 'collapsibleModule collapsibleModuleActive';
        var imagename = img_path + 'images/collapse-icon.png';
	
        for(i = 0; i < images.length; i++) { 
    		var source = images[i].src;
	    	    if(source.indexOf(img_path + "images/expand-icon.png") > -1) {
		    	images[i].src = imagename;
		    }
	    }
    }
}
