/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2011-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: log-search.js 11469 2020-04-29 20:34:29Z deiony $
 */

// TODO: can we use strict mode?
//"use strict";

// set up an object as way of namespacing.
// TODO: I recall there was some discussion about whether to use this style...
//   should we stop? I think there's mostly benefits and not much changed
//   from using regular globals.
if (!MS) {
    var MS = {};
}
if (!MS.Logs) {
    MS.Logs = {};
}

//! whether we've run our page on load callback already.
// TODO: this should not really be necessary unless we include this javascript
//   file twice, I think... but just in case...
MS.Logs.onLoadHasRun = false;

//! the callback function triggered by logSearchAjax
/*
 * @param object res the response object passed in by logSearchAjax
 *
 * @return void
 *
 * @author Sean West <sean@linuxmagic.com>
 * @author Peter Chung <peterc@linuxmagic.com>
 *
 * TODO: this function appears unused.
 */
function logSearchCallback(res) {
    var logresults_form = document.getElementById("logresults_form");
    var logresults_output = document.getElementById("logresults_output");
    var overlay = document.getElementById("overlay");
    logresults_form.style.display = "block";
    overlay.style.display    = "block";
    logresults_output.innerHTML     = res.responseText;
}

//! trigger by error
/*
 * @param string str the error string
 *
 * @return void
 *
 * @author Sean West <sean@linuxmagic.com>
 * @author Peter Chung <peterc@linuxmagic.com>
 *
 * SIDE EFFECT: Generates an alert
 *
 * TODO: this function appears unused.
 */
function logSearchErrorCallback(str) {
    // TODO: encoding/escpaing of the string?
    alert("logSearchCallback: "+str);
}

//! close the result overlay
/*
 * @return void
 *
 * @author Sean West <sean@linuxmagic.com>
 * @author Peter Chung <peterc@linuxmagic.com>
 *
 * TODO: this function seems unused... what attaches it does not actually
 *   show up in the interface...
 */
function closePopup() {
    var logresults_form = document.getElementById("logresults_form");
    var overlay = document.getElementById("overlay");
    logresults_form.style.display = "none";
    overlay.style.display    = "none";
}

function logSearchToggleAuthUserFilter(mua_select) {
    // sanity check
    if (!mua_select) {
        return;
    }

    // ensure log search authenticated user filter input exists
    var auth_user_filter = mua_select.previousSibling.previousSibling;
    if (!auth_user_filter || auth_user_filter.id != "auth_addr") {
        return;
    }

    // set the input status according to the mua select value,
    // enable when (0 = both, 1 = yes) and disable when (2 = no)
    auth_user_filter.disabled = (mua_select.value == 2);
}

//! ajax log search javascript
/*
 * @param string module the calling module
 * @param string submodule the sub module
 * @param string action the action that will be perform
 *
 * @return void
 *
 * @author Sean West <sean@linuxmagic.com>
 * @author Peter Chung <peterc@linuxmagic.com>
 *
 * TODO: this function appears unused!
 */
function logSearchAjax(module, controller) {
    var submodule    = "logSearch";
    var action       = "search";
    var from_addr    = document.getElementById("from_addr").value;
    var rcpt_addr    = document.getElementById("rcpt_addr").value;
    var result_limit = document.getElementById("result_limit").value;
    var filter       = document.getElementById("filter").value;
    var data_string  = "module=" + module + "&submodule=" + submodule;
    data_string     += "&from_addr=" + from_addr + "&rcpt_addr=" + rcpt_addr;
    data_string     += "&result_limit="+result_limit+"&ms_action=" + action
    data_string     += "&output_only=1";
    data_string     += "&filter="+filter;
    ajaxRequest(controller, logSearchCallback, data_string, logSearchErrorCallback);
}

function logSearchAddCountryToList(cc, list, blurb, extra) {
    // sanity check
    if (!cc || !list || !blurb || !extra) {
        return;
    }

    // only operate on blacklsit
    if (list !== "black") {
        return;
    }

    // change black to their proper lists
    list += "_country";

    // send to the IP function
    logSearchAddIPToList(cc, list, blurb, extra);
}

//! Ajax Whitelist / Blacklist Sender function
/*
 * @param   sender the IP address
 * @param   list   the given list (e.g. Whitelist / Blacklist) 
 * @param   blurb  confirm dialogue 
 *
 * @return void 
 * 
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function logSearchAddSenderToList(entry, list, blurb) {
    
    // Basic validation 
    if (!entry || !list || !blurb) {
        return;
    }

    // Check if only white / black list
    if (list !== "white" && list !== "black" && 
    list !== "white_domain") {
        return;
    }

    // Change white / black to their proper lists
    list += "_sender";

    // Send to the IP function
    logSearchAddIPToList(entry, list, blurb);
}

//! Ajax Whitelist / Blacklist IP function
/*
 * @param  res     the server response
 *
 * @return string  explains whether or not the entry was added
 * 
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function logSearchAddToListCB(page_request) {

    // Basic validation
    if (!page_request.responseText) {
       return; 
    }

    // Variable that will store our response as an array
    var responseArray; 

    // Parse the errors / notices
    if (typeof JSON == 'undefined') {
        // Work-around for the dreadfully hopeless IE7
        responseArray = eval("[" + page_request.responseText + "]"); 
    } else {
        responseArray = JSON.parse("[" + page_request.responseText + "]"); 
    }

    // If the response array doesn't have errors/notices, then return
    if (!responseArray[0].errors || !responseArray[0].notices) {
       return;
    }

    // logSearch.php should have DIV elements for errors / notices
    var errorBox = document.getElementById("errors");
    var noticeBox = document.getElementById("notices");

    // Gotta do a sanity check
    if (!errorBox || !noticeBox) {
        return;
    }

    // If errors, display them...
    if (responseArray[0].errors !== "no_errors") {

        // Set the style and content of the box 
        errorBox.style.display = "block";
        errorBox.innerHTML = responseArray[0].errors;
    
    // Else clear away any existing content
    } else {

        // Set the style and content of the box 
        errorBox.style.display = "none";
        errorBox.innerHTML = "";
    }

    // If notices, display them...
    if (responseArray[0].notices !== "no_notices") {
        
        // Set the style and content of the box 
        noticeBox.style.display = "block";
        noticeBox.innerHTML = responseArray[0].notices;
    
    // Else clear away any existing content
    } else {

        // Set the style and content of the box 
        noticeBox.style.display = "none";
        noticeBox.innerHTML = "";
    }
}

//! Ajax Whitelist / Blacklist IP function
/*
 * @param   ip     the IP address
 * @param   list   the given list (e.g. Whitelist / Blacklist) 
 *
 * @return void 
 * 
 * XXX: change function name.. this is not strictly for IP addresses...
 */
function logSearchAddIPToList(ip, list, blurb, extra) {
	 
    // Basic validation 
    if (!ip || !list || !blurb) {
        return;
    }

    // Ask the user if they are certain they wish to do this...
    if (!extra) {
        var areYouSure = confirm( blurb + ip + "?");
    } else {
        var areYouSure = confirm( blurb + ip + " (" + extra + ")?");
    }

    // If confirmed, then the magic happens...
    if (areYouSure) {

        // Variable declaration
        var ajax_data_string = "";
        var list_type = "";
        var action = "";
        var params = "";
        
        // Determines the list to add the IP to:
        //    list = white for Whitelist
        //    list = black for Blacklist
        if (list == "white") { 
            list_type = "ipWhiteList";
            action = "add";
            params = "&new_entry=" + ip;
        } else if (list == "white_sender") {
            list_type = "fromWhiteList";
            action = "add";
            params = "&new_entry=" + ip;
        } else if (list == "white_domain_sender") {
            list_type = "fromWhiteList";
            action = "add";
            params = "&new_entry=*@" + ip;
        } else if (list == "black") {
            list_type = "ipBlackList";
            action = "add";
            params = "&new_entry=" + ip;
        } else if (list == "black_sender") {
            list_type = "fromBlackList";
            action = "add";
            params = "&new_entry=" + ip;
        } else if (list == "black_country") {
            list_type = "countryBlackList";
            action = "add";
            params = "&new_entry=" + ip;
        } else if (list == "remove_white") {
            list_type = "ipWhiteList";
            action = "delete";
            params = "&entry[" + ip + "]=on";
        } else if (list == "remove_black") {
            list_type = "ipBlackList";
            action = "delete";
            params = "&entry[" + ip + "]=on";
        } else if (list == "white_ratelimit_inbound") {
            list_type = "ipRLWhiteList";
            action = "add";
            params = "&new_entry=" + ip;

            // we must do a secondary request to unblock them
            doSecondaryRequest("ipRateLimitList", "unblock", ip);

        } else if (list == "white_ratelimit_outbound") {
            list_type = "userRLWhiteList";
            action = "add";
            params = "&new_entry=" + ip;

            // we must do a secondary request to unblock them
            doSecondaryRequest("userRateLimitList", "unblock", ip);
        } else {
            // Otherwise some junk value has been inserted here, so we must return
            return;
        }

        // If we've gotten this far, go ahead and close the menu
        if (document.getElementById("MS_logsearch_menu") !== null) {
            MS.closeMenu('MS_logsearch_menu');
        } 

        // Assembles the data string
        if (MS.isFrontend) {
            ajax_data_string  = "module=feExemption";
        } else {
            ajax_data_string  = "module=settings";
            ajax_data_string += "&page=exemption";
        }

        ajax_data_string += "&submodule=" + list_type;
        ajax_data_string += "&ms_action=" + action;
        ajax_data_string += params;
        ajax_data_string += "&output_only=1";

        // Sends an AJAX request to add an IP address to the proper list 
        ajaxRequest(location.pathname.replace('logsearch', 'index'), logSearchAddToListCB, ajax_data_string); 
    }
}

function doSecondaryRequest(module, action, value) {
    ajax_data_string  = "module=" + module;
    ajax_data_string += "&ms_action=" + action;
    ajax_data_string += "&value=" + value;
    ajax_data_string += "&is_ajax=1";

    ajaxRequest(location.pathname.replace('logsearch', 'index'), logSearchAddToListCB, ajax_data_string);
}

//! Ajax Replay Mail Session function
/*
 * @param   row   the mail session information
 */
function logSearchRunReplay(row) {
    // sanity check
    if (typeof(row) !== 'object' || row.length !== 11) {
        return;
    }

    // the row contains the following mail session information
    //   Index :   0     1     2     3      4      5    6       7      8     9     10
    //   Format: <time><type><rule><mua><authuser><ip><rdns><country><helo><from><rcpt>
    // remove the time, type, rule, authuser and country data since the replay utility
    // does not need them
    row.splice(7, 1);
    row.splice(4, 1);
    row.splice(0, 3);

    // build the payload for the AJAX call
    var payload = "module=replay";
    var header = ["mua", "ip", "rdns", "helo", "from", "rcpt"];
    for (var i = 0; i < header.length; i++) {
        payload += "&" + encodeURI(header[i]) + "=" + encodeURI(row[i]);
    }
    payload += "&ms_action=replay";
    payload += "&output_only=1";
    payload += "&is_ajax=1";

    // don't forget to close the menu
    MS.closeMenu("MS_logsearch_menu");

    // make the AJAX call
    ajaxRequest(location.pathname.replace('logsearch', 'index'), logSearchRunReplayCB, payload, logSearchErrorReplayCB);
}

//! the callback function to the AJAX Replay Mail Session
/*
 * @param   data   the Replay Mail Session response
 */
function logSearchRunReplayCB(data) {
    // sanity check
    if (typeof(data) !== "object") {
        return;
    }

    // grab the replay popup html element
    var replayBlock = document.getElementById("replay-popup");
    if (replayBlock === null) {
        return;
    } 
   
    // inject and display the html from the replay module
    replayBlock.innerHTML = data.responseText;
    replayBlock.style.display = "block";
}

//! the error callback function to the AJAX Replay Mail Session
/*
 * @param   error   the error message
 */
function logSearchErrorReplayCB(error) {
    // sanity check
    if (typeof(error) !== "string") {
        return;
    }

    // inject the html from the module into the page
    var errorBlock = document.getElementById("errors");
    if (errorBlock === null) {
        return;
    } 
   
    errorBlock.innerHTML = error;
    errorBlock.style.display = "block";
}

//! hides the replay popup html element
function hideReplay() {
    // grab the replay popup html element
    var replayBlock = document.getElementById("replay-popup");
    if (replayBlock === null) {
        return;
    } 

    // hide the popup
    replayBlock.style.display = "none";
}

//! ajax CSV downloader
/*
 * @param string module The calling module
 * @param string submodule The sub module
 * @param string action The action that will be perform
 * @param string from_addr Address of the sender. Search pattern. May be blank.
 * @param string rcpt_addr Address of the receiver. Search pattern.
 *   May be blank.
 * @param string ip_address Search string for 'IP address'. May be blank.
 * @param int filter The filter option applied. This is one of the
 *   integers defined in logSearchModule: FILTER_*.
 *
 * @return void
 *
 * @author Sean West <sean@linuxmagic.com>
 * @author Peter Chung <peterc@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * SIDE EFFECT: Submits a form.
 *
 * NOTE: All string functions should be raw (non-HTML).
 */
function downloadCSV(module, submodule, action, from_addr, rcpt_addr, auth_addr, mua_status,
    ip_address, start_date, end_date, filter, limit, hour, yesterday, formHandler) {
    // sanity checks.
    if (typeof module !== 'string' || module.length === 0
        || typeof submodule !== 'string' || submodule.length === 0
        || typeof action !== 'string' || action.length === 0
        || typeof from_addr !== 'string'
        || typeof rcpt_addr !== 'string'
        || typeof auth_addr !== 'string'
        || typeof mua_status !== 'number'
        || typeof ip_address !== 'string'
        || typeof start_date !== 'string'
        || typeof end_date !== 'string'
        || typeof limit !== 'number'
        || typeof filter !== 'number' || filter < 0
        || typeof hour !== 'number'
        || typeof yesterday !== 'number') {
        return;
    }
    var browsertypes = css_browser_selector(navigator.userAgent);
    isIE = browsertypes.indexOf("ie ");

    if (isIE > -1) {
        isIE = 1;
    }
    var form = document.createElement("form");
    form.setAttribute("method", "POST");
    if (formHandler && formHandler !== "") {
        form.setAttribute('action', formHandler);
    } else {
        form.setAttribute("action", "index.php");
    }
    var hiddenFields = {
        "module":       module,
        "submodule":    submodule,
        "ms_action":    action,
        "from_addr":    from_addr,
        "rcpt_addr":    rcpt_addr,
        "auth_addr":    auth_addr,
        "mua_status":   mua_status,
        "ip_address":   ip_address,
        "start_date":   start_date,
        "end_date":     end_date,
        'result_limit': limit,
        "filter":       filter,
        "ms_hour":      hour,
        "yesterday":    yesterday,
        // output_only means we do not create the HTML document upon
        // response.
        "output_only": 1,
        "ie" : isIE
    };

    // XXX: little hack for InterWorx functionality
    if (typeof magicspam_siteworx_user !== "undefined") {
        hiddenFields.magicspam_siteworx_user = magicspam_siteworx_user;
    }

    for (var key in hiddenFields) {
        if (!hiddenFields.hasOwnProperty(key)) {
            continue;
        }
        var hiddenField = document.createElement("input");
        hiddenField.setAttribute("type", "hidden");
        hiddenField.setAttribute("name", key);
        hiddenField.setAttribute("value", hiddenFields[key]);
        form.appendChild(hiddenField);
    }

    document.body.appendChild(form);
    form.submit();
    document.body.removeChild(form);
}

function downloadAuthCSV(module, submodule, action, auth_addr, ip_address, start_date, end_date, filter, limit, formHandler) {
    // sanity check
    if (typeof module !== 'string' || module.length === 0
        || typeof submodule !== 'string' || submodule.length === 0
        || typeof action !== 'string' || action.length === 0
        || typeof auth_addr !== 'string'
        || typeof ip_address !== 'string'
        || typeof start_date !== 'string'
        || typeof end_date !== 'string'
        || typeof filter !== 'number' || filter < 0
        || typeof limit !== 'number') {
        return;
    }

    // determine whether the browser type is Internet Explorer
    var browsertypes = css_browser_selector(navigator.userAgent);
    isIE = browsertypes.indexOf("ie ");
    if (isIE > -1) {
        isIE = 1;
    }

    // create the form element for the POST
    var form = document.createElement("form");
    form.setAttribute("method", "POST");
    if (formHandler && formHandler !== "") {
        form.setAttribute('action', formHandler);
    } else {
        form.setAttribute("action", "index.php");
    }

    // initialize the key-value pairs for the input
    var hiddenFields = {
        "module"       : module,
        "submodule"    : submodule,
        "ms_action"    : action,
        "auth_addr"    : auth_addr,
        "ip_address"   : ip_address,
        "start_date"   : start_date,
        "end_date"     : end_date,
        "filter"       : filter,
        'result_limit' : limit,
        "output_only"  : 1,
        "ie"           : isIE
    };

    // append all the input to the form
    for (var key in hiddenFields) {
        if (!hiddenFields.hasOwnProperty(key)) {
            continue;
        }
        var hiddenField = document.createElement("input");
        hiddenField.setAttribute("type", "hidden");
        hiddenField.setAttribute("name", key);
        hiddenField.setAttribute("value", hiddenFields[key]);
        form.appendChild(hiddenField);
    }

    // add, submit and remove the form
    document.body.appendChild(form);
    form.submit();
    document.body.removeChild(form);
}
//! trim whitespace from a string.
/*!
 * @params string str String to trim
 *
 * @returns string. If you provide invalid input then we will return a
 *   blank string.
 *
 * @author William Storey <william@linuxmagic.com>
 */
MS.Logs.trimString = function(str) {
    if (str === undefined || typeof str !== 'string') {
        MS.log("trimString: invalid parameter");
        return '';
    }
    var newStr = str.replace(/^\s+|\s+$/g, '');
    return newStr;
};

//! retrieve and sanitise criteria form input values.
/*!
 * @params HTMLFormElement criteriaForm The form.
 *
 * @return mixed Object with properties: sender, recipient, ip,
 *   limit (each corresponding to the inputs), or undefined if a failure
 *   occurred.
 *   Each will be a string, and trimmed.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * find the input elements.
 *
 * pull out the values, and trim them, and return them as an object.
 */
MS.Logs.getCriteriaFormValues = function(criteriaForm) {
    if (!criteriaForm) {
        MS.log("getCriteriaFormValues: invalid parameter");
        return undefined;
    }
    var senderInput = criteriaForm.elements.from_addr;
    if (!senderInput) {
        MS.log("getCriteriaFormValues: sender input missing");
        return undefined;
    }
    var recipientInput = criteriaForm.elements.rcpt_addr;
    if (!recipientInput) {
        MS.log("getCriteriaFormValues: recipient input missing");
        return undefined;
    }
    var authUserInput = criteriaForm.elements.auth_addr;
    if (!authUserInput) {
        MS.log("getCriteriaFormValues: auth user input missing");
        return undefined;
    }
    var ipInput = criteriaForm.elements.ip_address;
    if (!ipInput) {
        MS.log("getCriteriaFormValues: ip input missing");
        return undefined;
    }
    var limitInput = criteriaForm.elements.result_limit;
    if (!limitInput) {
        MS.log("getCriteriaFormValues: limit input missing");
        return undefined;
    }
    var startDateInput = criteriaForm.elements.start_date;
    if (!startDateInput) {
        MS.log("getCriteriaFormValues: start date input missing");
        return undefined;
    }
    var endDateInput = criteriaForm.elements.end_date;
    if (!endDateInput) {
        MS.log("getCriteriaFormValues: end date input missing");
        return undefined;
    }
    var sender = MS.Logs.trimString(senderInput.value);
    var recipient = MS.Logs.trimString(recipientInput.value);
    var auth_user = MS.Logs.trimString(authUserInput.value);
    var ip = MS.Logs.trimString(ipInput.value);
    var limit = MS.Logs.trimString(limitInput.value);
    var start_date = MS.Logs.trimString(startDateInput.value);
    var end_date = MS.Logs.trimString(endDateInput.value);
    return {
        'sender': sender,
        'recipient': recipient,
        'auth_user': auth_user,
        'ip': ip,
        'limit': limit,
        'start_date': start_date,
        'end_date': end_date,
    };
};

//! validate a log search criteria: address input type.
/*!
 * @params string value This should already be trimmed.
 *
 * @return bool Whether valid or not
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we expect this to be email address form, with possible wildcards.
 */
MS.Logs.validateCriteriaAddressType = function(value) {
    if (value === undefined || typeof value !== 'string') {
        MS.log("validateCriteriaAddressType: invalid parameter");
        return false;
    }
    // blank is OK. that means the search will not be restricted to this.
    if (value.length === 0) {
        return true;
    }
    // if there is a wildcard, then we don't enforce any validation. why?
    // because we have to allow for partial addresses then.
    if (value.indexOf('*') !== -1) {
        return true;
    }
    // no wildcards. this means we should have a valid email address.
    // TODO: we don't have a nice javascript function to check email address
    //   validity, so here I will write very basic validation so we at least
    //   have something.
    // there must be a single '@'.
    var pieces = value.split('@');
    if (pieces.length !== 2) {
        MS.log("validateCriteriaAddressType: must have a single @ character.");
        return false;
    }
    var name = MS.Logs.trimString(pieces[0]);
    var host = MS.Logs.trimString(pieces[1]);
    // there must be characters before and after the @.
    if (name.length === 0 || host.length === 0) {
        MS.log("validateCriteriaAddressType: name or host is blank");
        return false;
    }
    // there must be at least one '.' after the @ with characters on each side.
    if (!host.match(/\S+\.\S+/)) {
        MS.log("validateCriteriaAddressType: . with chars on either side"
            + " not found in host");
        return false;
    }
    // there must not be more than two '.' in a row on the host side.
    if (host.match(/\.\./)) {
        MS.log("validateCriteriaAddressType: .. found in host");
        return false;
    }
    // the host must not end with an '.'.
    if (host.match(/\.$/)) {
        MS.log("validateCriteriaAddressType: . character at end of host");
        return false;
    }
    // the name must not contain spaces
    if (name.match(/\s/)) {
        MS.log("validateCriteriaAddressType: space character present in name");
        return false;
    }
    // the host must not contain spaces
    if (host.match(/\s/)) {
        MS.log("validateCriteriaAddressType: space character present in host");
        return false;
    }
    return true;
};

//! validate a log search criteria: postoffice input type
/*!
 * @params string               the input to validate, should already be trimmed
 *
 * @return bool                 whether valid postoffice input
 *
 * @author Deion Yu <deiony@linuxmagic.com>
 *
 * NOTE: a postoffice address must not contain whitespace and the
 *       @:[]*?\/ characters, the domain portion does not need to
 *       be a FQDN, i.e. user@postoffice.com and user@postoffice
 *       are both valid
 */
MS.Logs.validateCriteriaPostofficeType = function(value) {
    // sanity check
    if (value === undefined || typeof value !== 'string') {
        MS.log("validateCriteriaPostofficeType: invalid parameter");
        return false;
    }

    // zero length input means that the filter won't be applied
    if (value.length === 0) {
        return true;
    }

    // skip validation when there's a wildcard character to allow for partial addresses
    if (value.indexOf('*') !== -1) {
        return true;
    }

    // ensure that the input consists of a user and domain portion separated
    // by the @ character
    var pieces = value.split('@');
    if (pieces.length !== 2) {
        MS.log("validateCriteriaPostofficeType: must have a single @ character.");
        return false;
    }

    // retrieve user and domain from input
    var name = MS.Logs.trimString(pieces[0]);
    var host = MS.Logs.trimString(pieces[1]);

    // ensure that the user and domain portion are not empty
    if (name.length === 0 || host.length === 0) {
        MS.log("validateCriteriaPostofficeType: name or host is blank");
        return false;
    }

    // ensure domain contain valid characters
    if (host.match(/[\s@:\[\]\*\?\\\/]/)) {
        MS.log("validateCriteriaPostofficeType: host cannot contain whitespace and @:[]*?\\\/ characters");
        return false;
    }

    // ensure user contain valid characters
    if (name.match(/\s/)) {
        MS.log("validateCriteriaPostofficeType: name cannot contain whitespace");
        return false;
    }
    return true;
};

//! validate a log search criteria: IP address input type.
/*!
 * @params string value The value should already be trimmed.
 *
 * @return bool Whether valid
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we expect this to be a valid ip with possible wildcards.
 *
 * this allows both IPv4 and IPv6.
 */
MS.Logs.validateCriteriaIPType = function(value) {
    if (value === undefined || typeof value !== 'string') {
        MS.log("validateCriteriaIPType: invalid parameter");
        return false;
    }

    // blank is OK. that means the search will not be restricted to this.
    if (value.length === 0) {
        return true;
    }

    // if we have both . and : then it is an invalid value.
    // that indicates a mixture of ipv4 and ipv6.
    if (value.match(/\./) && value.match(/:/)) {
        MS.log("validateCriteriaIPType: mixed IPv4 and IPv6 address separators found");
        return false;
    }

    // if there is a wildcard then we use less strict validation.
    if (value.indexOf('*') !== -1) {
        return MS.Logs.isValidWildcardIP(value);
    }

    // with no wildcards we require a full address.

    // is this an ipv4 or an ipv6 address?

    // ipv6 has colon separators. if we see one, treat it as ipv6.
    if (value.match(/:/)) {
        return MS.Logs.isValidIPv6IP(value);
    }

    // otherwise it must be ipv4.

    // character check: only . and digits permitted.
    if (value.match(/[^.0-9]/)) {
        MS.log("validateCriteriaIPType: bad ipv4 character found");
        return false;
    }

    // format check.
    if (!value.match(/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/)) {
        MS.log("validateCriteriaIPType: bad ipv4 form found");
        return false;
    }

    // TODO: we could check range of octets too ... e.g. no '777.1.2.3'.

    return true;
};

//! check if a log search address value - wildcard -is valid
/*!
 * @params string value A wildcard value.
 *
 * @return bool Whether valid
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * @pre We have trimmed the value
 * @pre We have detected it is a wildcard type input.
 * @pre The value is not blank.
 * @pre The value has been checked to not have mixed ipv4/ipv6 separators.
 *
 * it may be ipv4 or ipv6.
 *
 * if there is a wildcard, then we don't enforce as strict validation.
 * why? because we have to allow for partial addresses then. we no longer
 * have full valid addresses.
 *
 * at this time we enforce only that there be only the expected characters.
 */
MS.Logs.isValidWildcardIP = function(value) {
    if (value === undefined) {
        MS.log("isValidWildcardIP: invalid parameter");
        return false;
    }

    // ipv6.
    if (value.match(/:/)) {
        if (value.match(/[^:0-9a-fA-F*]/)) {
            MS.log("isValidWildcardIP: ipv6: bad character found in wildcard");
            return false;
        }
        return true;
    }

    // ipv4.
    if (value.match(/\./)) {
        if (value.match(/[^.0-9*]/)) {
            MS.log("isValidWildcardIP: ipv4: bad character found in wildcard");
            return false;
        }
        return true;
    }

    // it's possible there is no separator (.:).
    // if so, restrict to the characters of both types of addresses.
    if (value.match(/[^0-9a-fA-F*]/)) {
        MS.log("isValidWildcardIP: ipv4/ipv6: bad character found in wildcard");
        return false;
    }

    return true;
};

//! check if a log search address value - ipv6 -is valid
/*!
 * @params string value The address
 *
 * @return bool Whether valid
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * @pre We have trimmed the value
 * @pre We have detected it is a wildcard type input.
 * @pre The value is not blank.
 * @pre The value has been checked to not have mixed ipv4/ipv6 separators.
 * @pre The value is not wildcard.
 */
MS.Logs.isValidIPv6IP = function(value) {
    if (value === undefined) {
        MS.log("isValidIPv6IP: invalid parameter");
        return false;
    }

    // check we have only the characters we expect.
    if (value.match(/[^:0-9a-fA-F]/)) {
        MS.log("isValidIPv6IP: invalid character found");
        return false;
    }

    // we can have consecutive sections of zeroes replaced with '::'.
    // if it happens then it must only happen once.
    if (value.match(/::.*?::/)) {
        MS.log("isValidIPv6IP: more than one double colon found");
        return false;
    }

    // more than 2 :: at once is invalid
    if (value.match(/:::/)) {
        MS.log("isValidIPv6IP: more than two colons found in a row");
        return false;
    }

    // cannot start and end with : (except for :: which I think TODO is valid)
    if (value.match(/^:.*?:$/)) {
        if (value === '::') {
            return true;
        }
        MS.log("isValidIPv6IP: starts and ends with colons");
        return false;
    }

    // if there is a collapsed sections of zeroes we apply a different check
    // than for a full address.
    if (value.match(/::/)) {
        // 1. it can start with ::.
        // 2. it can end with ::. (TODO: I think)
        // 3. or it is in the middle.
        //
        // a). each group must have at most 4 characters and at minimum 1.
        // b). there must be at most 7 groups since we collapsed at least 1.
        // c). there must be at least 0 groups (TODO: assuming :: means 0).

        // to know how many groups I split it first on the ::.
        // then I check each group's characters individually.
        // partly because I want to count how many character groups there are.
        var sections = value.split("::");
        if (sections.length !== 2) {
            MS.log("isValidIPv6IP: unexpected number of collapsed sections");
            return false;
        }

        // break up into individual groups now.
        // we can have a blank part if we had :: at the beginning or the end.
        var groups = [];
        if (sections[0] !== '') {
            var groups0 = sections[0].split(":");
            groups = groups.concat(groups0);
        }
        if (sections[1] !== '') {
            var groups1 = sections[1].split(":");
            groups = groups.concat(groups1);
        }

        var groupCount = 0;
        for (var i = 0; i < groups.length; i++) {
            var group = groups[i];
            // an empty group is invalid at this point.
            // because this would mean we had something like
            // ^:abc or abc:$. the only blank after the split should happen
            // at the collapsed section.

            if (!group.match(/^[0-9a-fA-F]{1,4}$/)) {
                MS.log("isValidIPv6IP: group has invalid character: " + group);
                return false;
            }
            groupCount++;
        }

        if (groupCount > 7) {
            MS.log("isValidIPv6IP: collapsed group has too many groups");
            return false;
        }
        if (groupCount === 0) {
            MS.log("isValidIPv6IP: no groups in collapse");
            return false;
        }

        return true;
    }

    // addresses that are not collapsed (with no ::) must have 8 groups.
    if (!value.match(/^(?:[0-9a-fA-F]{1,4}:){7}[0-9a-fA-F]{1,4}$/)) {
        MS.log("isValidIPv6IP: invalid address");
        return false;
    }

    return true;
};

//! validate a log search criteria: limit input type.
/*!
 * @params string value The value should be trimmed.
 *
 * @returns bool Whether valid
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we require this to be a positive integer.
 * here we will not enforce any upper bound however.
 */
MS.Logs.validateCriteriaLimitType = function(value) {
    if (value === undefined || typeof value !== 'string') {
        MS.log("validateCriteriaLimitType: invalid parameter");
        return false;
    }
    // must be all digits.
    if (!value.match(/^[1-9]+[0-9]*$/)) {
        MS.log("validateCriteriaLimitType: invalid character found: [" + value + "]");
        return false;
    }
    return true;
};

//! determine whether the date is valid
/*!
 * @params  number day    the day of the date
 * @params  number month  the month of the date
 * @params  number year   the year of the date
 *
 * @returns bool whether the date is valid
 *
 * @author  Deion Yu <deiony@linuxmagic.com>
 */
MS.Logs.isDateValid = function(day, month, year) {
    // sanity check
    if (day === undefined || typeof day !== 'number'
        || month === undefined || typeof month !== 'number'
        || year === undefined || typeof year !== 'number') {
        return false;
    }
    // determine the days in the month for the year
    var daysInMonth = 0;
    switch (month) {
        case 2:
            daysInMonth = (year % 4 == 0 && year % 100) || year % 400 == 0 ? 29 : 28;
            break;
        case 9: case 4: case 6: case 11:
            daysInMonth = 30;
            break;
        default:
            daysInMonth = 31;
    }
    // determine whether the date is valid
    return month > 0 && month <= 12 && day > 0 && day <= daysInMonth;
};

//! validate a log search criteria: date
/*!
 * @params  string              the trimmed date input (yyyy-mm-dd)
 *
 * @returns bool                whether the date input is valid
 *
 * @author  Deion Yu <deiony@linuxmagic.com>
 */
MS.Logs.validateCriteriaDateType = function(value) {
    // sanity check
    if (value === undefined || typeof value !== 'string') {
        MS.log("validateCriteriaDateType: invalid parameter");
        return false;
    }
    if (value.length == 0) {
        return true;
    }
    // initialize date regex
    var date_regex = /^([0-9]{4})\-([0-9]{1,2})\-([0-9]{1,2})/g;
    // validate date input
    var match = date_regex.exec(value);
    if (!match || !MS.Logs.isDateValid(+match[3], +match[2], +match[1])) {
        MS.log("validateCriteriaDateType: invalid character found: [" + value + "]");
        return false;
    }
    return true;
};

//! validate a log search criteria: date range
/*!
 * @params  string              the trimmed start date input (yyyy-mm-dd)
 * @params  string              the trimmed end date input (yyyy-mm-dd)
 *
 * @returns bool                whether the date input is valid
 *
 * @author  Deion Yu <deiony@linuxmagic.com>
 */
MS.Logs.validateCriteriaDateRangeType = function(start, end, errors) {
    // sanity check
    if (start === undefined || typeof start !== 'string'
        || end === undefined || typeof end !== 'string') {
        errors.push('logSearch.error.invalidDateCriteria');
        return false;
    }
    // ensure that the start and end dates are in the same state
    if ((start.length == 0) ^ (end.length == 0)) {
        errors.push('logSearch.error.invalidDateRangeCriteria');
        return false;
    }
    // ensure that both start and end dates are valid
    if (!MS.Logs.validateCriteriaDateType(start)
        || !MS.Logs.validateCriteriaDateType(end)) {
        errors.push('logSearch.error.invalidDateCriteria');
        return false;
    }
    // ensure that the start date is before the end date
    if (new Date(start) > new Date(end)) {
        errors.push('logSearch.error.invalidDateRangeCriteria');
        return false;
    }
    return true;
};

//! clear any errors we might be displaying.
/*!
 * @returns bool Whether successful
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we remove the div#errors all together to accomplish this.
 */
MS.Logs.clearDisplayedErrors = function() {
    // parent of the errors. we need it to
    var msWrapper = document.getElementById('magicspam-wrapper');
    if (!msWrapper) {
        MS.log("clearDisplayedErrors: div#magicspam-wrapper not found!");
        return false;
    }
    var errorContainer = document.getElementById('errors');
    // if it's not present in the document then there's nothing for us to do.
    if (!errorContainer) {
        return true;
    }
    // remove it all together to clear the errors. we don't want an empty
    // error container hanging around.
    msWrapper.removeChild(errorContainer);
    return true;
};

//! show errors about log search criteria.
/*!
 * @params Array errors Array of i18n codes to display as errors.
 *
 * @returns bool Whether successful
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we translate each code into a full string and insert it into the page.
 *
 * we do not need to remove displayed errors - that should already be dealt with
 * outside of this function. we simply append.
 */
MS.Logs.showCriteriaErrors = function(errors) {
    // try to determine this is actually an array.
    if (errors === undefined || typeof errors !== 'object' ||
        errors.length === undefined) {
        MS.log("showCriteriaErrors: invalid parameter");
        return false;
    }
    // nothing to show!
    if (errors.length === 0) {
        return true;
    }
    // if errors div exists, then we'll add to it. otherwise, create it.
    var errorContainer = document.getElementById('errors');
    var errorList;
    if (errorContainer) {
        errorList = errorContainer.firstChild;
        if (!errorList || errorList.tagName.toUpperCase() !== 'UL') {
            MS.log("showCriteriaErrors: div#errors exists but has no <ul>?");
            return false;
        }
    } else {
        errorContainer = document.createElement('div');
        errorContainer.setAttribute('id', 'errors');
        // add a close button.
        var closeButton = document.createElement('span');
        errorContainer.appendChild(closeButton);
        var closeButtonA = document.createElement('a');
        // TODO: dynamic label?
        closeButtonA.innerHTML = 'X';
        closeButtonA.setAttribute('href', '#');
        MS.addElementOnClick(closeButtonA, function(evt) {
            // don't have the href be followed.
            if (evt) {
                // some versions of IE don't have preventDefault...
                if (evt.preventDefault) {
                    evt.preventDefault();
                } else {
                    evt.returnValue = false;
                }
            }
            MS.Logs.clearDisplayedErrors();
            return false;
        });
        closeButton.appendChild(closeButtonA);
        var errorList = document.createElement('ul');
        errorContainer.appendChild(errorList);
        // add it into the document.
        var msWrapper = document.getElementById('magicspam-wrapper');
        if (!msWrapper) {
            MS.log("showCriteriaErrors: div#magicspam-wrapper not found!");
            return false;
        }
        // add it as its first child so we're the first child!
        var msWrapperChild = msWrapper.firstChild;
        // if there's somehow no child, then just add in at the end.
        if (!msWrapperChild) {
            msWrapper.appendChild(errorContainer);
        } else {
            msWrapper.insertBefore(errorContainer, msWrapperChild);
        }
    }
    for (var i = 0; i < errors.length; i++) {
        MS.log("showCriteriaErrors: showing error: " + errors[i]);
        var errorItem = document.createElement('li');
        // intl() text must be HTML encoded.
        errorItem.innerHTML = MS.intl(errors[i]);
        errorItem.className = "log_search_error";
        errorList.appendChild(errorItem);
    }
    return true;
};

//! event handler for criteria form submit event. validate.
/*!
 * @param nsiDOMEvent evt
 *
 * @return bool True if everything is fine, false otherwise.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we validate each field on the form.
 *
 * if it is invalid, we show an error message in the page, and do not
 * permit the form to be submitted, as in that case we do not want to create
 * a new tab if possible.
 *
 * if everything is valid we will allow the form to be submitted.
 *
 * note we are not trying to have perfect validation here but merely assist
 * for usability.
 *
 * inputs to validate:
 * - sender: from address also allowing wildcards
 * - recipient: to address also allowing wildcards
 * - ip address: ip address also allowing wildcards
 * - filter: this should not be able to be changed by a sane client
 *   as it is a dropdown, so no validation at this time.
 * - apply limit: this is a checkbox and should not be a problem for
 *   a sane client, so likewise leave it alone for now.
 * - limit: while this is enforced client side already to be valid,
 *   check anyway.
 */
MS.Logs.criteriaFormValidate = function(evt) {
    // ie8 apparently does not have event passed?
    if (!evt) {
        evt = window.event;
    }
    if (!evt) {
        MS.log("criteriaFormValidate: event missing");
        return false;
    }
    // ie8 apparently does not have event.target.
    var criteriaForm = evt.target;
    if (!criteriaForm) {
        criteriaForm = evt.srcElement;
    }
    if (!criteriaForm) {
        MS.log("criteriaFormValidate: event target missing so cannot find"
            + " form");
        return false;
    }
    // stop default action so we won't submit. we could do this when we first
    // enter the function, but after basic sanity checks may be a little nicer
    // in case there are any cross browser issues in that the form submission
    // may still be possible.
    // ie8 does not have preventDefault().
    if (evt.preventDefault) {
        evt.preventDefault();
    } else {
        evt.returnValue = false;
    }
    // pull out the form's data.
    var values = MS.Logs.getCriteriaFormValues(criteriaForm);
    if (!values) {
        MS.log("criteriaFormValidate: values not found");
        return false;
    }
    // run our checks. store up error codes in an array.
    var errors = [];
    // check sender: address type.
    if (!MS.Logs.validateCriteriaAddressType(values.sender)) {
        errors.push('logSearch.error.invalidSenderCriteria');
    }
    // check recipient: address type.
    if (!MS.Logs.validateCriteriaAddressType(values.recipient)) {
        errors.push('logSearch.error.invalidRecipientCriteria');
    }
    // check auth user: address type as email address
    if (!MS.Logs.validateCriteriaAddressType(values.auth_user)) {
        // check auth user: address as postoffice when MailEnable
        if (MS.suite === undefined || typeof(MS.suite) !== 'string'
            || MS.suite.indexOf('MailEnable') === -1
            || !MS.Logs.validateCriteriaPostofficeType(values.authUser)) {
            errors.push('logSearch.error.invalidAuthUserCriteria');
        }
    }
    // check ip address: ip type.
    if (!MS.Logs.validateCriteriaIPType(values.ip)) {
        errors.push('logSearch.error.invalidIPCriteria');
    }
    // check limit: integer type.
    if (!MS.Logs.validateCriteriaLimitType(values.limit)) {
        errors.push('logSearch.error.invalidLimitCriteria');
    }
    // check date: date range type
    MS.Logs.validateCriteriaDateRangeType(values.start_date, values.end_date, errors);

    // if we had any errors, then show them in the page, and don't submit.
    if (errors.length > 0) {
        MS.log("criteriaFormValidate: error detected");
        MS.Logs.clearDisplayedErrors();
        MS.Logs.showCriteriaErrors(errors);
        return false;
    }
    // XXX: Workaround for DirectAdmin Evolution Skin, will not affect anything else
    if (criteriaForm.action.indexOf('?iframe=yes') != -1)  {
        criteriaForm.action = criteriaForm.action.split('?')[0];
    }
    // everything checks out, so submit the form. we stopped default action
    // so otherwise the form won't submit.
    MS.log("criteriaFormValidate: validated!");
    criteriaForm.submit();
    // also ensure we clear out any displayed errors.
    MS.Logs.clearDisplayedErrors();
    return true;
};

MS.Logs.authLogSearchFormValidate = function(evt) {
    // ie8 apparently does not have event passed?
    if (!evt) {
        evt = window.event;
    }
    if (!evt) {
        MS.log("criteriaFormValidate: event missing");
        return false;
    }
    // ie8 apparently does not have event.target.
    var criteriaForm = evt.target;
    if (!criteriaForm) {
        criteriaForm = evt.srcElement;
    }
    if (!criteriaForm) {
        MS.log("criteriaFormValidate: event target missing so cannot find"
            + " form");
        return false;
    }
    // stop default action so we won't submit. we could do this when we first
    // enter the function, but after basic sanity checks may be a little nicer
    // in case there are any cross browser issues in that the form submission
    // may still be possible.
    // ie8 does not have preventDefault().
    if (evt.preventDefault) {
        evt.preventDefault();
    } else {
        evt.returnValue = false;
    }
    // pull out the form's data.
    var authUserInput = criteriaForm.elements.auth_addr;
    if (!authUserInput) {
        MS.log("getCriteriaFormValues: auth user input missing");
        return undefined;
    }
    var ipInput = criteriaForm.elements.ip_address;
    if (!ipInput) {
        MS.log("getCriteriaFormValues: ip input missing");
        return undefined;
    }
    var limitInput = criteriaForm.elements.result_limit;
    if (!limitInput) {
        MS.log("getCriteriaFormValues: limit input missing");
        return undefined;
    }
    var startDateInput = criteriaForm.elements.start_date;
    if (!startDateInput) {
        MS.log("getCriteriaFormValues: start date input missing");
        return undefined;
    }
    var endDateInput = criteriaForm.elements.end_date;
    if (!endDateInput) {
        MS.log("getCriteriaFormValues: end date input missing");
        return undefined;
    }
    var values = {
        'authUser': MS.Logs.trimString(authUserInput.value),
        'ip': MS.Logs.trimString(ipInput.value),
        'limit': MS.Logs.trimString(limitInput.value),
        'start_date': MS.Logs.trimString(startDateInput.value),
        'end_date': MS.Logs.trimString(endDateInput.value),
    };
    // run our checks. store up error codes in an array.
    var errors = [];
    // check auth user: address type as email address
    if (!MS.Logs.validateCriteriaAddressType(values.authUser)) {
        // check auth user: address as postoffice when MailEnable
        if (MS.suite === undefined || typeof(MS.suite) !== 'string'
            || MS.suite.indexOf('MailEnable') === -1
            || !MS.Logs.validateCriteriaPostofficeType(values.authUser)) {
            errors.push('logSearch.error.invalidAuthUserCriteria');
        }
    }
    // check ip address: ip type.
    if (!MS.Logs.validateCriteriaIPType(values.ip)) {
        errors.push('logSearch.error.invalidIPCriteria');
    }
    // check limit: integer type.
    if (!MS.Logs.validateCriteriaLimitType(values.limit)) {
        errors.push('logSearch.error.invalidLimitCriteria');
    }
    // check date: date range type
    MS.Logs.validateCriteriaDateRangeType(values.start_date, values.end_date, errors);

    // if we had any errors, then show them in the page, and don't submit.
    if (errors.length > 0) {
        MS.log("criteriaFormValidate: error detected");
        MS.Logs.clearDisplayedErrors();
        MS.Logs.showCriteriaErrors(errors);
        return false;
    }
    // XXX: Workaround for DirectAdmin Evolution Skin, will not affect anything else
    if (criteriaForm.action.indexOf('?iframe=yes') != -1)  {
        criteriaForm.action = criteriaForm.action.split('?')[0];
    }
    // everything checks out, so submit the form. we stopped default action
    // so otherwise the form won't submit.
    MS.log("criteriaFormValidate: validated!");
    criteriaForm.submit();
    // also ensure we clear out any displayed errors.
    MS.Logs.clearDisplayedErrors();
    return true;
};

//! on load event for the log search criteria page.
/*!
 * @return void
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we ensure we are actually on the main log search criteria page (as I believe
 * this javascript may be included in more locations than there).
 *
 * we attach our validation callbacks to the form inputs.
 */
MS.Logs.onLoadCriteriaPage = function() {
    // flag that we've run to not run multiple times. see the declaration
    // of this property.
    if (MS.Logs.onLoadHasRun) {
        MS.log("onLoadCriteriaPage: on load has already run");
        return;
    }
    MS.Logs.onLoadHasRun = true;
    // ensure we're on the log search criteria page.
    // do this by looking for its container div.
    var containerDiv = document.getElementById('logSearchContainer');
    if (!containerDiv) {
        MS.log("onLoadCriteriaPage: container not found");
        return;
    }
    // find our form.
    var criteriaForm = document.forms.logsearch;
    if (!criteriaForm) {
        MS.log("onLoadCriteriaPage: form not found");
        return;
    }
    // attach our callback validation event.
    if (!MS.addElementOnSubmit(criteriaForm, MS.Logs.criteriaFormValidate)) {
        MS.log("onLoadCriteriaPage: failed to attach submit callback");
        return;
    }
    // optionally find our auth log search form
    var authLogSearchForm = document.forms.authlogsearch;
    if (authLogSearchForm) {
        // attach our callback validation event
        if (!MS.addElementOnSubmit(authLogSearchForm, MS.Logs.authLogSearchFormValidate)) {
            MS.log("onLoadCriteriaPage: failed to attach submit callback");
            return;
        }
    }
    MS.log("onLoadCriteriaPage: attached submit callback");
};

//! run some unit tests.
/*!
 * @returns void
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * TODO: this probably doesn't belong here...
 */
MS.Logs.test = function() {
    MS.log("Logs.test: starting tests");

    var success = true;
    var validAddressCriteria = [
        'test*',
        '*',
        '*test*',
        'te*st',
        '',
        'test@linuxmagic.com'
    ];
    for (var i = 0; i < validAddressCriteria.length; i++) {
        var value = validAddressCriteria[i];
        if (!MS.Logs.validateCriteriaAddressType(value)) {
            MS.log("Logs.test: valid address criteria said to be invalid: "
                + value);
            success = false;
        }
    }
    var invalidAddressCriteria = [
        '@',
        't@',
        'test@@linuxmagic.com',
        '@hi',
        'test@hi',
        'test@.hi',
        'test@linuxmagic..com',
        'test@linuxmagic.com.',
        '@linuxmagic.com'
    ];
    for (var i = 0; i < invalidAddressCriteria.length; i++) {
        var value = invalidAddressCriteria[i];
        if (MS.Logs.validateCriteriaAddressType(value)) {
            MS.log("Logs.test: invalid address criteria said to be valid: "
                + value);
            success = false;
        }
    }

    var validIPCriteria = [
        '*',
        '192*',
        '*192',
        '192.168.1.1',
        '::1',
        // TODO: I am not sure about this one.
        '::',
        '1::',
        '2001:0db8:0000:0000:0000:ff00:0042:8329',
        '2001:db8:0:0:0:ff00:42:8329',
        '2001:db8::ff00:42:8329',
        '1:2:3::4:5:6',
        '1:2:3::4:5:6:7',
        'ffff::1',
        'dead::beef',
        '123::b:c:d'
    ];
    for (var i = 0; i < validIPCriteria.length; i++) {
        var value = validIPCriteria[i];
        if (!MS.Logs.validateCriteriaIPType(value)) {
            MS.log("Logs.test: valid IP criteria said to be invalid: "
                + value);
            success = false;
        }
    }

    var invalidIPCriteria = [
        '192',
        '192..168.1.1',
        '192.168.1.',
        'abc',
        '1',
        '192.abc.*',
        '1::2::1',
        '1:2:3',
        '1:2:3:4:5:6:7',
        '1:2:3:4:5:6:7:8:9',
        '1:2:3:4:5:6:7:8:',
        '1:2:3:4:5:6:7:',
        ':1:2:3:4:5:6:7',
        ':0:1:2:3:4:5:6:7',
        '1.3:4.5',
        '1.*:1',
        'fffg::1',
        '1:2:3::4:5:6:7:8',
        '1:2:3:::4:5:6:7:8',
        'dead:::beef',
        'dead:::1::beef',
        'dead:::1:beef',
        'deade::beef',
        ':dead::beef:',
        ':1:2:3:4:5:6:7:8:',
        '1:2:3:4:5:6:7:8:',
        ':1:2:3:4:5:6:7:8',
        ':123::bcd',
        '123::bcd:'
    ];
    for (var i = 0; i < invalidIPCriteria.length; i++) {
        var value = invalidIPCriteria[i];
        if (MS.Logs.validateCriteriaIPType(value)) {
            MS.log("Logs.test: invalid IP criteria said to be valid: "
                + value);
            success = false;
        }
    }
    var validLimitCriteria = [
        '1',
        '2',
        '50',
        '500'
    ];
    for (var i = 0; i < validLimitCriteria.length; i++) {
        var value = validLimitCriteria[i];
        if (!MS.Logs.validateCriteriaLimitType(value)) {
            MS.log("Logs.test: valid limit criteria said to be invalid: "
                + value);
            success = false;
        }
    }
    var invalidLimitCriteria = [
        '0',
        '-1',
        'abc'
    ];
    for (var i = 0; i < invalidLimitCriteria.length; i++) {
        var value = invalidLimitCriteria[i];
        if (MS.Logs.validateCriteriaLimitType(value)) {
            MS.log("Logs.test: invalid limit criteria said to be valid: "
                + value);
            success = false;
        }
    }
    if (success) {
        MS.log("Logs.test: tests ran successfully");
    } else {
        MS.log("Logs.test: tests encountered a failure");
    }
};

MS.Logs.onLoadInitToggleAndResize = function() {
    // lock down the resize functionality to the log search result page
    // NOTE: the #logresults of the Log Snip has #magicspam-wrapper as a parent,
    //       while the #logresults of the Log Seach Results does not
    if (document.querySelector('#magicspam-wrapper #logresults')) {
        return;
    }

    // the HTML elements that control the total width of table
    tables = document.querySelectorAll('#logresults, .rowTable');

    // the HTML elements for each header in the table
    headers = document.querySelectorAll('#logresults div[class^="header_"]');

    // the HTML elements for each resize cell in the table
    resizes = document.querySelectorAll('#logresults div[class^="resize_"]');

    // the HTML elements for each row in the table
    rows = document.querySelectorAll('.oddRow, .evenRow');

    // the HTML elements for each cell in the table grouped by columns
    // NOTE: will be populated as needed in the mousedown event
    columns = {};

    // the widths to toggle between for each column
    // NOTE: will be populated as needed in both the click and mousedown events
    columns_toggle_widths = {};

    // the resize helper variables
    resizeMode = false;
    column = "";
    startPosX = 0;
    startColumnWidth = 0;
    startTableWidth = 0;

    // helper functions for toggling and resizng, not needed in global scope
    // set the user-select style for a list of nodes
    setUserSelectStyle = function(nodes, value) {
        for (i = 0; i < nodes.length; i++) {
            nodes[i].style.WebkitTouchCallout = value;
            nodes[i].style.WebkitUserSelect = value;
            nodes[i].style.KhtmlUserSelect = value;
            nodes[i].style.MozUserSelect = value;
            nodes[i].style.MsUserSelect = value;
            nodes[i].style.userSelect = value;
        }
    }
    // retrieves the list of cells within the column
    retrieveColumnsCells = function(column) {
        // only initialize once per column as the number of cells in the column
        // is expected to be constant
        if (!(column in columns)) {
            // build the classes of the cells within the column
            headerClass = ".header_" + column;
            logClass = ".log_" + column;

            // store the cells in the column
            columns[column] = document.querySelectorAll(headerClass + ', ' + logClass);
        }
        // return the cells in the column
        return columns[column];
    }
    // retrieves the mouse x position
    getMousePositionX = function(e) {
        e = e || window.event;
        posX = e.pageX;
        if (posX === undefined) {
            // Internet Explorer Workaround
            posX = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
        }
        return posX;
    }

    // set the onclick event on each of the table headers to toggle the column
    for (i = 0; i < headers.length; i++) {
        headers[i].onclick = function(e) {
            // retrieve the column name
            column = e.target.classList[0].replace("header_", "");

            // retrieve all the cells for the column
            cells = retrieveColumnsCells(column);

            // calculate the new toggle and table width
            if (cells[0].offsetWidth > 20) {
                newToggleWidth = 20;
                newTableWidth = tables[0].offsetWidth - (cells[0].offsetWidth - 20);

                // additionally store the current width of the column whenever we
                // are wider than the minimum width
                columns_toggle_widths[column] = cells[0].offsetWidth;
            } else {
                newToggleWidth = columns_toggle_widths[column];
                newTableWidth = tables[0].offsetWidth + (columns_toggle_widths[column] - 20);
            }

            // toggle the width of all the cells in the column
            for (i = 0; i < cells.length; i++) {
                cells[i].style.width = newToggleWidth + "px";
            }

            // set the width of the table to the new table width
            for (i = 0; i < tables.length; i++) {
                tables[i].style.width = newTableWidth + "px";
            }
        };
    }

    // set the onmousedown event on each of the table resize cells to start of resizing
    for (i = 0; i < resizes.length; i++) {
        resizes[i].onmousedown = function(e) {
            // store the starting mouse position
            startPosX = getMousePositionX(e);

            // store the column name
            column = e.target.classList[0].replace("resize_", "");

            // initialize the array of cells in the column as needed
            cells = retrieveColumnsCells(column);

            // store the starting width of the column and table
            startColumnWidth = cells[0].offsetWidth;
            startTableWidth = tables[0].offsetWidth;

            // disable text select temporarily while resizing
            setUserSelectStyle(rows, "none");

            // set resize mode to active
            resizeMode = true;
        };
    }

    // set the onmouseup event to reset for the next resize
    document.onmouseup = function(e) {
        // don't run when not in resize mode
        if (resizeMode !== true) {
            return;
        }

        // re-enable text select now that resizing is complete
        setUserSelectStyle(rows, "auto");

        // reset the resize helper variables
        resizeMode = false;
        column = "";
        startPosX = 0;
        startColumnWidth = 0;
        startTableWidth = 0;

    };

    // set the onmousemove event to resize the table columns
    document.onmousemove = function(e) {
        // don't run when not in resize mode
        if (resizeMode !== true) {
            return;
        }

        // get the current mouse position
        curPosX = getMousePositionX(e);

        // calculate difference in x position
        diffPosX = startPosX - curPosX;

        // calculate the new width of the column
        newColumnWidth = startColumnWidth - diffPosX;
        if (newColumnWidth < 20) {
            newColumnWidth = 20;
        }

        // column width has not changed, nothing more to do
        if (newColumnWidth == startColumnWidth) {
            return;
        }

        // calculate the new table width
        newTableWidth = startTableWidth - diffPosX;
        if (newTableWidth < startTableWidth - startColumnWidth + 20) {
            newTableWidth = startTableWidth - startColumnWidth + 20;
        }

        // update the toggle width where the next toggle will set the width to
        // the resized width so long as the column was resized to more than the
        // minimum column width, otherwise the next toggle will use the original
        // width of the column before the resize
        if (newColumnWidth > 20) {
            columns_toggle_widths[column] = newColumnWidth;
        } else {
            columns_toggle_widths[column] = startColumnWidth;
        }

        // set the width of each cell to the new width in the header column
        cells = retrieveColumnsCells(column);
        for (i = 0; i < cells.length; i++) {
            cells[i].style.width = newColumnWidth + "px";
        }

        // set the width of the table to the new table width
        for (i = 0; i < tables.length; i++) {
            tables[i].style.width = newTableWidth + "px";
        }
    };
}

MS.Logs.onLoadDateInput = function() {
    var startDateInput = document.getElementById('start_date');
    var endDateInput = document.getElementById('end_date');
    if (startDateInput && endDateInput) {
        startDateInput.onchange = function() {
            if (!endDateInput.value) {
                endDateInput.value = startDateInput.value;
            }
            endDateInput.min = startDateInput.value;
        }
        endDateInput.onchange = function() {
            if (!startDateInput.value) {
                startDateInput.value = endDateInput.value;
                endDateInput.min = endDateInput.value;
            }
        }
    }

    var authStartDateInput = document.getElementById('auth_start_date');
    var authEndDateInput = document.getElementById('auth_end_date');
    if (authStartDateInput && authEndDateInput) {
        authStartDateInput.onchange = function() {
            if (!authEndDateInput.value) {
                authEndDateInput.value = authStartDateInput.value;
            }
            authEndDateInput.min = authStartDateInput.value;
        }
        authEndDateInput.onchange = function() {
            if (!authStartDateInput.value) {
                authStartDateInput.value = authEndDateInput.value;
                authEndDateInput.min = authEndDateInput.value;
            }
        }
    }
}

MS.addDocumentOnLoad(MS.Logs.onLoadDateInput);
MS.addDocumentOnLoad(MS.Logs.onLoadCriteriaPage);
MS.addDocumentOnLoad(MS.Logs.onLoadInitToggleAndResize);
