/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: migration.js 10940 2020-01-14 00:44:10Z deiony $
 */

// set up a namespace to try and avoid naming collisions
if (!MS) {
    var MS = {};
}

//! sends POST request to export migration file
MS.exportMigrationFile = function() {
    // enable the migration ie input if necessary
    var browsertypes = css_browser_selector(navigator.userAgent);
    if (browsertypes.indexOf("ie ") > -1) {
        document.querySelector('#migration-form input[name="ie"]').disabled = false;
    }

    // prepare the migration form
    MS.prepareMigrationForm('export', false);

    // submit the migration form
    // NOTE: only submit the form directly during export for the sake of cross
    //       browser compatability
    var form = document.getElementById('migration-form');
    form.submit();
}

//! sends POST request to import the migration file
MS.importMigrationFile = function() {
    // retrieve the browser type and file input
    var browsertypes = css_browser_selector(navigator.userAgent);
    var file_input = document.getElementById("migration-file-input");

    // display loading element to give user visual feedback
    showLoading();

    // prepare the migration form
    MS.prepareMigrationForm('import', true);

    // trigger file input to prompt user for migration file when firefox
    // NOTE: firefox won't focus a file input with the click of a
    //       corresponding label
    if (browsertypes.indexOf("firefox ") > -1) {
        file_input.click();
    }

    // NOTE: do not directly trigger the submission of the migration form
    //       when import for the sake of cross browser compatability, we
    //       use an event triggered by a label connected to the file input
    //       to get around the pesky IE 'access denied' error
    //       https://stackoverflow.com/questions/9396411/ie-javascript-form-submit-with-file-input
}

//! prepares and submits the migration form
/*!
 * @param   action         the type of migration action
 * @param   output_only    whether to disable output only input
 */
MS.prepareMigrationForm = function(action, output_only) {
    // set the migration action input
    var action_input = document.querySelector('#migration-form input[name="ms_action"]');
    action_input.value = action;

    // disable the output only input
    var output_only_input = document.querySelector('#migration-form input[name="output_only"]');
    output_only_input.disabled = output_only;
}

//! registers events needed for migration
MS.registerMigrationEvents = function() {
    // retrieve the migration file input
    var file_input = document.getElementById("migration-file-input");

    // submit the migration form only after file input has changed during import
    file_input.onchange = function(e) {
        document.getElementById('migration-form').submit();
    };
}

// register the migration events
MS.addDocumentOnLoad(MS.registerMigrationEvents);
