/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2011-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: rate-limiter.js 6173 2017-01-25 17:24:37Z shaun $
 */

//! utility function used by 'remove' links in rate limited ip table to initialize the actual removal.
/*!
 * @param   string addr the ip address to remove.
 *
 * @return none
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * SIDE EFFECTS: addr not being set properly will cause the function to return early.
 */
function unblockIP(addr) {
    if (!addr || !addr.length) {
        return;
    }
    document.body.style.cursor = "wait";
    document.forms['rateLimiterListForm'].ip.value = addr;
    document.forms['rateLimiterListForm'].submit();
}

//! form submitter for unblocking multiple entries
/*
 * @param string form_id the form id
 *
 * @author Robert Bisewski <robertb@linuxmagic.com> 
 */
function unblockMulti(form_id) {
    var form   = document.getElementById(form_id);
    var action = document.forms[form_id].ms_action;
    if (!form || !action) {
        alert(MS.intl('rl_unblock_failed'));
        return;
    }

    // Confirms deletion
    if (!confirm(MS.intl('rl_unblock_multiple'))) {
        return;
    }
    document.body.style.cursor = "wait";
    // Submits the form
    form.submit();
}

//! creates a form for the Rate Limiter List submodules
/*!
 * @param string various entries in the form
 *
 * @return none
 *
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function doFormSubmitRL(params) {

    var form = document.createElement("form");
    form.setAttribute("method", "POST");
    form.setAttribute("action", "");

    // Confirms unblocking, otherwise ends...
    if (params["hiddenFields"]["ms_action"] != undefined 
            && params["hiddenFields"]["ms_action"] == 'unblock'
            && !confirm(MS.HTMLToText(MS.intl('rl_unblock_entry')) + params["hiddenFields"]["value"] + " ?")) {
        return;
    }

    // Grabs the hidden fields and appends them to the form, otherwise ends...
    if (params["hiddenFields"] != undefined) {
        for (var key in params["hiddenFields"]) {
            var hiddenField = document.createElement("input");
            hiddenField.setAttribute("type", "hidden");
            hiddenField.setAttribute("name", key);
            hiddenField.setAttribute("value", params["hiddenFields"][key]);
            form.appendChild(hiddenField);
        }
    } else {
        return;
    }
    document.body.style.cursor = "wait";
    // Attaches and submits the form
    document.body.appendChild(form);
    form.submit();
    document.body.removeChild(form);
}
