/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019-2021 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: repair-permission.js 12182 2021-02-18 01:32:55Z deiony $
 */

// set up a namespace to try and avoid naming collisions
if (!MS) {
    var MS = {};
}

//! Sends an AJAX request to repair permission
/*!
 * @param   action    the type of action to repair permission
 */
MS.repairPermissionAjax = function(action, url) {
    // display loading element to give user visual feedback
    showLoading();

    // retrieve URL for submitting the AJAX request     
    if (!url) {
        url = window.location.pathname;
    }

    // XXX: little hack for the Interworx URL     
    if (url.indexOf('nodeworx') === 1) {     
        url = window.location.protocol + '//' + window.location.host + '/plugins/images/magicspampro/index.php';  
    }

    // build the data string and send the AJAX request
    var data_string = "module=packages&ms_action=repair_permission_" + action + "&output_only=1";
    ajaxRequest(url, MS.repairPermissionCallback, data_string, MS.repairPermissionErrorCallback);
}

//! Callback function when AJAX to repair permission request succeeds
/*!
 * @param   res       the AJAX request response object
 */
MS.repairPermissionCallback = function(res) {
    // hide the loading element
    hideLoading();

    // sanity check
    if (!res || !res.response || res.response == '') {
        return;
    }

    // retrieve the popup element
    var popup = document.getElementById('repair-permission-popup');
    if (!popup) {
        return;
    }

    // display popup with the content of response
    popup.innerHTML = res.response;
    popup.style.display = "block";
}

//! Callback function when AJAX to repair permission request fails
/*!
 * @param   str       the AJAX request response text
 */
MS.repairPermissionErrorCallback = function(str) {
    // hide the loading element
    hideLoading();

    // display the error message
    MS.buildNotice(str, "errors");
}

//! Hides the repair permission popup
MS.repairPermissionClose = function() {
    // retrieve the popup element
    var popup = document.getElementById('repair-permission-popup');
    if (!popup) {
        return;
    }

    // hide the popup
    popup.style.display = "none";
}
