/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2013-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: server-policies.js 11290 2020-03-27 16:26:44Z deiony $
 */

if (!MS) {
    var MS = {};
}

//! utility function to reset spam rules to default recommended
/*!
 * @authors Anonymous
 *
 * NOTE: Bound to button click handler.
 */
function setServerPoliciesToDefault()
{
    document.forms['serverPoliciesForm'].ms_action.value
        = 'set_server_policies_to_defaults';
    document.forms['serverPoliciesForm'].submit();
}

//! primary onclick function to start the rejection message loading process.
/*!
 * @param   string  the rule name to load
 *
 * @author Anonymous
 *
 * SIDE EFFECTS: this function does not return but rather starts
 *               an ajax call with callback return to editRuleCB()
 */
function editRule(rule, url) {
    ruleName = rule;
    document.forms['rejectionEditor'].rule.value = rule;
    if (!url) {
        url = window.location.pathname;
    }
    ajaxRequest(url, editRuleCB,
        "output_only=1&module=serverPolicies&ms_action=load_rejection&rule="
        + rule);
}

//! callback to display the rejection message editor.
/*!
 * @author Anonymous
 */
function editRuleCB(page_request) {
    document.body.style.cursor = 'default';
    var obj = document.getElementById('rejectionForm');
    var rname = document.getElementById('rejectionRuleName');
    var tfield = document.getElementById('rejectionNotice');
    if (!obj || !rname || !tfield) {
        MS.log("editRuleCB: form / rule / notice not found");
        return;
    }

    tfield.value = page_request.responseText;
    rname.innerHTML = ruleName;
    obj.style.display = 'block';
    limitRejectionNotice(document.forms['rejectionEditor']);

    // center the form
    // TODO: should we make top calculation based on window size too?
    obj.style.top = document.body.offsetHeight / 2 - 200 + 'px';
    // determine where to place horizontally based on the body width.
    var xPos = document.body.offsetWidth / 2;
    // and adjust it left by half of our form's width.
    xPos -= obj.offsetWidth / 2;
    if (xPos < 0) {
        xPos = 0;
    }
    obj.style.left = xPos + 'px';

    tfield.focus();
}

// simple function to hide the form.
/*!
 * @author Anonymous
 */
function cancelRejection() {
    document.getElementById('rejectionForm').style.display = 'none';
}

//! utility function to update the 'characters remaining' field and ensure newlines don't get added to the mix.
/*!
 * @param   jsOBJ   the primary form wrapper to operate on.
 * @author Anonymous
 */
function limitRejectionNotice(theForm) {
    // update the characters remaining
    var curText = theForm.notice.value;

    var cleanText = curText.replace(/\n/g,'');
    curText = cleanText.replace(/\r/g,'');
    var len = curText.length;

    if (len > 512) {
        newText = curText.substring(0,512);
        curText = newText;
        theForm.notice.value = curText;
        len = 512;
    }
    document.getElementById('rejectionCharsLeft').value = 512-len;
}

//! event handler to disable 'Enter' key in custom rejection message editor
/*!
 * @param   eventOBJ    the 'event' that the HTML document produced
 *
 * @returns boolean     false = do not process event
 *                      true  = process event as normal.
 * @author Anonymous
 */
function disableKeys(e) {
    var key = e ? e.which : window.event.keyCode;
    var node = (e.target) ? e.target : ( (e.srcElement) ? e.srcElement : null);
    if (node.id == 'rejectionNotice') {
        if (key == 13 || key == 34 || key == 38 || key == 39) {
            return false;
        }
    }
    return true;
}

//! call back function to lock out the Enter key when the rejection form is active.
/*!
 * NOTE: there is no 'unset' equivalent. Because disableEnter
 *       is set to also identify based on active object, there is no need.
 * @author Anonymous
 */
function limitEnter() {
    document.onkeypress=disableKeys;
}

function swapClasses(arr, swapfrom, swapto) {
    for (var i = 0; i < arr.length; i++) {
        if (arr[i].className == swapfrom) {
            arr[i].className = swapto;
        }
    }
}

//! Shows / hides the first two rows (BMS and RBL) of the Server Policies table
/*!
 * @author Robert Bisewski
 */
function toggleBlockLists(action) {

    // Determines if an action was actually given...
    if (!action) {
        return;
    }

    // safety check to make sure sanity of 'checked' box matches with 
    // action.  NOTE: this may also preclude the need to have
    // an argument to this function at all!
    var toggleBox = document.getElementById('toggleBox');
    if (toggleBox) {
        if (toggleBox.checked) {
            action = 'show';
        } else {
            action = 'hide';
        }
    }

    // Attempts to grab the first two rows, and checks if they exist...
    var forms = document.getElementById('serverPolicies');
    if (forms === null) {
        return;
    }    
    forms = forms.getElementsByTagName('form');
    if (forms === null) {
        return;
    }
    var rowTables = forms[0].getElementsByTagName('div');
    if (!rowTables) {
        return;
    }
    var rows = rowTables[0].getElementsByTagName('div');
    if (!rows || rows.length < 1) {
        return;
    }

    // Searchs for the first oddRow and evenRow, then shows / hides them
    var hasFirstRowHidden = false;
    var hasSecondRowHidden = false;
    for (var i = 0; i < rows.length; i++) {
        if (action == 'hide' && rows[i].className == 'oddRow' && !hasFirstRowHidden && !hasSecondRowHidden) {
            rows[i].className = 'oddRowHidden';
            hasFirstRowHidden = true;
        } else if (action == 'hide' && rows[i].className == 'evenRow' && hasFirstRowHidden && !hasSecondRowHidden) {
            rows[i].className  = 'evenRowHidden';
            hasSecondRowHidden = true;
        } else if (action == 'show' && rows[i].className == 'oddRowHidden') {
            rows[i].className = 'oddRow';
        } else if (action == 'show' && rows[i].className == 'evenRowHidden') {
            rows[i].className = 'evenRow';
        }
    }
}

function toggleRejections(checkbox) {
    if (!checkbox) {
        return;
    }
    if (checkbox.checked == 1) {
        showRejections();
        toggleBlockLists('show');
    } else {
        hideRejections();
        toggleBlockLists('hide');
    }
}

function showRejections() {
    // parse through all of the 'spans' and 'divs'
    var spans = document.getElementsByTagName('span');
    var divs = document.getElementsByTagName('div');

    // in order to swap out the classes.
    swapClasses(spans, 'ruleDesc', 'ruleDescSmall');
    swapClasses(divs, 'ruleDesc', 'ruleDescSmall');
    swapClasses(spans, 'ruleActions', 'ruleActionsVisible');
    swapClasses(divs, 'ruleActions', 'ruleActionsVisible');
    swapClasses(spans, 'recommendation', 'recommendationSmall');
    swapClasses(divs, 'recommendation', 'recommendationSmall');
}

function hideRejections() {

    var spans = document.getElementsByTagName('span');
    var divs = document.getElementsByTagName('div');

    swapClasses(spans, 'ruleDescSmall', 'ruleDesc');
    swapClasses(divs, 'ruleDescSmall', 'ruleDesc');
    swapClasses(spans, 'ruleActionsVisible', 'ruleActions');
    swapClasses(divs, 'ruleActionsVisible', 'ruleActions');
    swapClasses(spans, 'recommendationSmall', 'recommendation');
    swapClasses(divs, 'recommendationSmall', 'recommendation');
}

// XXX - do these belong here?  Specific to RBL management but added to server policies?????

//! opens the RBL help popup via the ? icon
/*
 * @param caller the form making the call
 *
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function showRBLHelp(caller) {
    if (!caller) {
        return;
    }
    var mypopup = document.getElementById('RBLHelp');
    if (!mypopup) {
        return;
    }
    var clientRect = caller.getBoundingClientRect();
    var bodyRect = document.body.getBoundingClientRect();
    var realTop = clientRect.top - bodyRect.top - 180;
    var realLeft = clientRect.left - bodyRect.left - 225;

    mypopup.style.left = realLeft + 'px';
    mypopup.style.top = realTop + 'px';
    mypopup.style.display = 'block';
}


//! closes the RBL help popup via the X icon
/*
 * @author Robert Bisewski <robertb@linuxmagic.com>
 */
function hideRBLHelp() {
    var mypopup = document.getElementById('RBLHelp');
    if (!mypopup) {
        return;
    }
    mypopup.style.display = 'none';
}

// attach our callbacks.
// what we do here:
// - KSF slider - show a notification
// - on load we may show the KSF notification (if we just set default
//   recommended policies)
MS.addDocumentOnLoad(function() {
    // Hides the BMS and RBL rows
    if (document.getElementById('toggleBox')) {
        toggleBlockLists('hide');
    }
    // find elements we need.
    var actionFlag = document.getElementById('last_action');
    if (!actionFlag) {
        MS.log("server-policies on load: action flag not found");
        return;
    }
    var ruleInput = MS.findRuleInput('known_sender_forgery');
    if (!ruleInput) {
        MS.log("server-policies on load: rule input not found");
        return;
    }

    var ksfSliderObj = MS.findRuleSlider('known_sender_forgery');
    if (!ksfSliderObj) {
        MS.log("server-policies on load: KSF slider not found");
        return;
    }

    var helpContentI18n = 'ksfWarningMessage';
    // if last action was set recommended rules, including KSF, show a help
    // bubble.
    if (ruleInput.value === 'true'
        && actionFlag.value === 'set_server_policies_to_defaults') {
        // HACK: we found that the positioning is slightly off (too far down)
        //   if we show this immediately on document load. I suspect it has
        //   something to do with hidden elements (or those that hide at
        //   document load somehow). as a work around we currently have a minor
        //   delay to show it which allows positioning to then work.
        //   this is a hack as we should not have to have this delay, but until
        //   we can dedicate time to figuring out exactly what's going on, let's
        //   go with this.
        var intervalId = window.setInterval(function() {
                MS.showHelpPopup(ksfSliderObj.parentNode, 'ksfCloseLabel',
                    helpContentI18n, undefined, undefined, 'left');
                window.clearInterval(intervalId);
            }, 100);
    }

    // attach onclick event to slider's div
    MS.addElementOnClick(ksfSliderObj.parentNode, function(e) {
        // remove pop-up if slider is set off
        if (ruleInput.value === 'false') {
            // hide the popup if it's present if we turn the rule off.
            MS.hideHelpPopup(helpContentI18n);
            return;
        }
        // if we are here then we want to show the popup - however...
        // the showHelpPopup has a bad habit of just toggling the popup
        // if it is already shown. so... we check if it exists first.
        // NOTE: this is expressly for the purpose of working around odd
        //       behavior when we have tri part sliders in PRO.
        var popupId = MS.getHelpPopupId(helpContentI18n);
        if (popupId) {
            var popup = document.getElementById(popupId);
            // it exists? nothing to do then.
            if (popup) {
                return;
            }
        }
        // if we are here, then the popup is not shown yet so.. show it
        MS.showHelpPopup(this, 'ksfCloseLabel', helpContentI18n, undefined,
            undefined, 'left');
    });

});
