/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2014-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: system.js 9857 2019-06-20 23:51:45Z deiony $
 */

//! utility function to switch between basic and advanced mode
/*!
 *  @return none
 */
function toggleAdvanced() {
    // retrieve all the toggable system options rows and horizontal lines
    var rows = document.querySelectorAll('#systemOptions div[id^="row"]');
    var hrs = document.querySelectorAll('#systemOptions hr[id^="hr"]');

    // toggle display of the system options rows
    for (var i = 0; i < rows.length; i++) {
        if (rows[i].className === "invisibleRow") {
            rows[i].className = "visibleRow";
        } else if (rows[i].className === "visibleRow") {
            rows[i].className = "invisibleRow";
        }
    }

    // toggle display of the system options horizontal lines
    for (var i = 0; i < hrs.length; i++) {
        if (hrs[i].className === "invisibleBar") {
            hrs[i].className = "visibleBar";
        } else if (hrs[i].className === "visibleBar") {
            hrs[i].className = "invisibleBar";
        }
    }
}

//! on load function to set focus on the admin email input IF we aren't editing
// a license AND the value is empty.
/*!
 * @returns void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 */
function focusAdminField() {
    var keyinput = document.getElementById('licenseKeyInput');
    if (keyinput) {
        MS.log('focusAdminField: key input is present');
        return;
    }
    if (!document.forms) {
        MS.log('focusAdminField: no forms found');
        return;
    }
    var systemOptionsForm = document.forms.sysOpts;
    if (!systemOptionsForm) {
        MS.log('focusAdminField: system options form not found');
        return;
    }
    var adminInput = systemOptionsForm.elements.email_address;
    if (!adminInput) {
        MS.log('focusAdminField: admin input not found');
        return;
    }
    var val = adminInput.value;
    if (val.length === 0) {
        MS.log('focusAdminField: admin input is blank');
        adminInput.focus();
    }
}

addLoadEvent(focusAdminField);
