"use strict"

/*
 * @param   mixed   The results of the ajax data request
 *
 * Creates the stats graph 
 */
function insertGraph(data) {
    if (data === undefined || typeof data !== 'object') {
        return;
    }

    // parse the AJAX response into JSON
    // NOTE: XMLHttpRequest.response is only available on IE10 and above,
    //       so fallback to XMLHttpRequest.responseText 
    var response = JSON.parse(data.response || data.responseText);
    if (typeof response !== 'object') {
        return;
    }

    var config = {
        type: 'line',
        data: {
            labels: response.timestamps,
            datasets: [{
                tension: "0",
                label: MS.i18n["spam"],
                backgroundColor : "rgba(255, 0, 0, 0.3)",
                data: response.spam,
            }, {
                tension: "0",
                label: MS.i18n["inbound"],
                backgroundColor : "rgba(0, 0, 255, 0.3)",
                data: response.inbound,
            }, {
                tension: "0",
                label: MS.i18n["outbound"],
                backgroundColor : "rgba(100, 0, 255, 0.3)",
                data: response.outbound,
            },{
                tension: "0",
                label: MS.i18n["ham"],
                backgroundColor : "rgba(0, 255, 0, 0.3)",
                data: response.ham,
                
            }

    ]
        },
        options: {
            responsive: true,
            title:{
                display:false,
                text: MS.i18n['chartTitle']
            },
            tooltips: {
                mode: 'index',
                intersect: false,
            },
            hover: {
                mode: 'nearest',
                intersect: true
            },
            scales: {
                xAxes: [{
                    display: true,
                    scaleLabel: {
                        display: true,
                        labelString: MS.i18n['hour']
                    }
                }],
                yAxes: [{
                    display: true,
                    scaleLabel: {
                        display: true,
                        labelString: MS.i18n['numMsg']
                    }
                }]
            }
        }
    };

    var ctx = document.getElementById("canvas").getContext("2d");
    window.myLine = new Chart(ctx, config);

    insertSummary(response);
}

/*
 * Inserts the summary data.
 *
 * @param   mixed   The results of the ajax data request
 */
function insertSummary(data) {
    if (data === undefined || typeof data.summary !== 'object') {
        return;
    }
    var summary = data.summary;

    document.getElementById("summary-inbound").innerHTML  = summary.inbound;
    document.getElementById("summary-outbound").innerHTML = summary.outbound;
    document.getElementById("summary-spam").innerHTML     = summary.spam;
    document.getElementById("summary-total").innerHTML    = summary.total;
    document.getElementById("summary-average").innerHTML  = summary.average;
}

/*
 * Performs an Ajax request for the hourly stat data
 *
 * @param   mixed   The results of the ajax data request
 */
function requestHourlyData() {
     // Assembles the data string
     var ajax_data_string = "";
     ajax_data_string  = "module=stats";
     ajax_data_string += "&page=stats";
     ajax_data_string += "&submodule=todayStats";
     ajax_data_string += "&ms_action=getHourlyStats";

     // Sends an AJAX request to add an IP address to the proper list 
     ajaxRequest("/?", insertGraph, ajax_data_string); 

}

document.addEventListener('DOMContentLoaded', requestHourlyData);
