<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: lib.globaltoggle.php 4221 2014-11-04 23:28:57Z shaun $
 *
 * This is a cli utilty to push a setting to all
 * magicspam users
 */
include_once('lib.cmdwrapper.php');

class GlobalToggle extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    const ENABLE = '-e';
    const DISABLE = '-d';
    const FLAG = '-f';

    //! static function to get the current singleton instance
    /*!
     * @returns GlobalToggle  the active singleton instance.
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     *
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new self(USER_RULE_MANAGER);
        }
        return self::$object;
    }

    //! togges a spam rule for all users
    /*!
     * @param string $spamRuleName the spam rule
     * @param string $option the option
     * @returns boolean whether it was successful
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     *
     */
    public function toggleSpamRule($spamRuleName, $option) {
        $buffer = "";
        if (!is_string($spamRuleName) || !strlen($spamRuleName)) {
            $this->lastError = intl("invalid_param");
            return FALSE;
        }

        if ($option !== self::ENABLE && $option !== self::DISABLE && $option !== self::FLAG) {
            $this->lastError = intl("invalid_param");
            return FALSE;
        }

        $args = array($option, '-s', $spamRuleName);

        $rv = $this->execRead($args, $buffer);

        if ($rv === 0) {
            return TRUE;
        }

        $this->lastError = intl("fail_sync");
        return FALSE;
    }

    //! toggles an ip list for all users
    /*!
     * @param string $ipRepListNum the spam rule
     * @param string $option the cli flag
     * @returns boolean whether it was successful
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     *
     */
    public function toggleIPRepList($ipRepListNum, $option) {
        $buffer = "";
        if (!is_numeric($ipRepListNum) || intval($ipRepListNum) < 0) {
            $this->lastError = intl("invalid_param");
            return FALSE;
        }

        if ($option !== self::ENABLE && $option !== self::DISABLE && $option !== self::FLAG) {
            $this->lastError = intl("invalid_param");
            return FALSE;
        }

        $args = array($option, '-i', $ipRepListNum);

        $rv = $this->execRead($args, $buffer);
        if ($rv === 0) {
            return TRUE;
        }

        $this->lastError = intl("fail_sync");
        return FALSE;
    }

}
