<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.ratelimitlist.php 6554 2017-08-02 22:24:51Z shaun $
 */
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('lib.rlmanager.php');
require_once('functions.php');

define('RLL_IP', 0);
define('RLL_USER', 1);

class rateLimitList extends Module {
    private $rlmanager;
    private $type;

    //! class constructor
    /*!
     * @param   string  the 'friendly' label for the module
     * @param   boolean whether or not the module should show up on the main menu
     * @param   integer the 'type' of list - based on defines above for IP or USER
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct($label, $menushow, $type) {
        global $ms_words;

        // pass up to the module constructor to init
        parent::__construct($label, $menushow);

        // grab the handle for the rate limiter manager
        $this->rlmanager = RateLimiterManager::getInstance();

        // default to 'ip' list.
        $this->type = RLL_IP;
        // then check and validate the argument
        if ($type == RLL_IP || $type == RLL_USER) {
            $this->type = $type;
        }
        // in this case we have to explicitly load the locale because we are using a derived common locale
        $moduleLocale = loadMSLocale('rateLimitList');
        if (file_exists($moduleLocale)) {
            require_once($moduleLocale);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $output_only;
        // no point in moving on if there is no action requested so...
        if (!is_string($action) || strlen($action) == 0) {
            return;
        }

       // sets the state of the collaspible modules	 
       if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
       }

        // Action processing begins here
        if ($action == 'unblock') {

            // Code for unblocking a single entry
            $val = isset($_POST['value']) ? $_POST['value'] : NULL;
            if ($val) {
                $fn = NULL;
                if ($this->type == RLL_IP) {
                   $fn = 'removeIP';
                }
                if ($this->type == RLL_USER) {
                    $fn = 'removeUser';
                }
                if ($fn == NULL) {
                    $errors[] = "attempt to process non initialized list type";
                } else {
                    if (!call_user_func(array($this->rlmanager, $fn), $val)) {
                        $errors[] = $this->rlmanager->error();
                    } else {
                        $notices[] = $val . " " . intl('removed');
                        // sleep a little to allow updates to propagate
                        sleep(3);
                    }
                }

            }

            // if its an ajax call, don't render the page
            if (isset($_POST['is_ajax']) && $_POST['is_ajax'] === '1') {
                $output_only = 1;
                return;
            }
        } else if ($action == 'unblock_multiple') {

            // Code for unblocking multiple entry
            $selectedEntries = isset($_POST['entry']) ? $_POST['entry'] : NULL;
            if ($selectedEntries) {
                $fn = NULL;
                if ($this->type == RLL_IP) {
                   $fn = 'removeIP';
                }
                if ($this->type == RLL_USER) {
                    $fn = 'removeUser';
                }
                if ($fn == NULL) {
                    $errors[] = intl('rl_unblock_failed');
                } else {

                    // Checks if any entries were selected...
                    if (count($selectedEntries) < 1) {
                        $notices[] = intl('rl_nothing_to_unblock');
                        return;
                    }

                    // Attempts to unblock each entry, ends if errors occur
                    foreach ($selectedEntries as $key => $selected) {
                        if (!call_user_func(array($this->rlmanager, $fn), $key)) {
                            $errors[] = $this->rlmanager->error();
                            return;
                        }
                    }

                    // If no errors / failures happen, sends a success notice
                    $notices[] = intl('rl_unblock_multiple_success');
                    sleep(3);
                }
            } else {
                // If $selectedEntries is NULL, assume there is nothing to unblock
                $notices[] = intl('rl_nothing_to_unblock');
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;
        $page = $title = $desc = $fn = $submodule = "";
        $expireTime = 0;

        // set up our rendering vars based on the type of list.
        if ($this->type == RLL_IP) {
            $page = 'ip_rl';
            $title = 'rl_list_title';
            $desc = 'rl_list_desc';
            $head1 = 'rl_ipaddress';
            $fn = 'listIPS';
            $submodule = 'ipRateLimitList';
            $expireTime = getControl("rl_block_timeout");
        }
        if ($this->type == RLL_USER) {
            $page = 'user_rl';
            $title = 'ra_list_title';
            $desc = 'ra_list_desc';
            $head1 = 'ra_user';
            $fn = 'listUsers';
            $submodule = 'userRateLimitList';
            $expireTime = getControl("ra_block_timeout");
        }

        // Grabs the unblock entry localization
        $this->_i18n_js[] = 'rl_unblock_entry';
        $this->_i18n_js[] = 'rl_unblock_failed';
        $this->_i18n_js[] = 'rl_unblock_multiple';

        // Setup our data table - common formatting for the most part...
        $table = new magicspamDataTable;
        $table->setClass("row", array('rl_fancyCheckBox', 'rl_ip','rl_date','rl_date', 'rl_remove'));
        $entries = call_user_func(array($this->rlmanager, $fn));

        // Attempts to sort the entries by IP address (lowest to highest)
        // NOTE: checking if defined here as apparently older PHP versions
        //   did not have this?
        if (defined('SORT_NATURAL')) {
            ksort($entries, SORT_NATURAL);
        } else {
            ksort($entries);
        }

        // set the full path for the 'trash' icon
        $trashicon = IMAGEPATH . "/icon-trashcan.png";

        // Adds the entries to the table
        if (is_array($entries)) {
            foreach ($entries as $ip => $ts) {
                $params = "{hiddenFields:{'ms_action':'unblock','value':'{$ip}'"
                        . ",'module':'{$module}','submodule':'{$submodule}',"
                        . "'page':'rate_limiter'}}";
                $table->addRow( array(
                                      "<input type=\"checkbox\" name=\"entry[" . htmlspecialchars($ip)  . "]\" />",
                                      htmlspecialchars($ip),
                                      date('D, d M Y H:i:s', $ts),
                                      date('D, d M Y H:i:s', $ts + $expireTime),
                                      "<a title=\"" . intl('unblock') . "\" onclick=\"doFormSubmitRL" .
                                      "(" . htmlspecialchars($params) . ");\"><img src=\"" .
                                      $trashicon . "\"/></a>"
                                     ));
            }
        }

        // Default: assumes table is empty
        $datastring = "<div class=\"smallTopMargin\">" . intl('no_entries') . "</div>";
        $showTableHeader = FALSE;

        // Otherwise: table has contents, so this displays them...
        if (is_array($entries) && sizeof($entries) > 0) {
            $datastring = $table->getRenderString();
            $showTableHeader = TRUE;
        }

        // If it is not the professional version, disable the outbound auth. rate limiter
        $authDisabled = FALSE;
        $isInbound = TRUE;
        if ($submodule == "userRateLimitList") {
            $isInbound = FALSE;
            if (!isProfessional()) {
                $authDisabled = TRUE;
            }
        }

        renderTemplate('rateLimitListModule', array (
                                                 'title'               => intl($title),
                                                 'desc'                => intl($desc),
                                                 'authDisabledDesc'    => intl('ra_list_authDisableDesc'),
                                                 'rl_form_name'        => "{$page}_form",
                                                 'rl_ip'               => intl('rl_ipaddress'),
                                                 'ra_user'             => intl('ra_user'),
                                                 'rl_date'             => intl('rl_dateblocked'),
                                                 'rl_expires'          => intl('rl_expires'),
                                                 'isInbound'           => $isInbound,
                                                 'authDisabled'        => $authDisabled,
                                                 'submodule'           => $submodule,
                                                 'showTableHeader'     => $showTableHeader,
                                                 'rl_unblock_selected' => intl('rl_unblock_selected'),
                                                 'table'               => $datastring,
                                               ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('ratelimiter-list.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('rate-limiter.js');
    }

}
