<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id: lib.repairuserpermissionsutility.php 9834 2019-06-18 19:21:10Z deiony $
 */
include_once('lib.cmdwrapper.php');

class RepairUserPermissionsUtility extends cmdWrapper {
    // internal protected singleton.
    protected static $object = NULL;

    //! static function to get the current singleton instance
    /*!
     * @returns object   the active singleton instance
     */
    static public function getInstance() {
        // the repair permissions utility needs root privileges
        if (is_null(self::$object)) {
            self::$object = new self(SECURE_BIN);
        }
        return self::$object;
    }

    public function runCheck(&$buffer) {
        return $this->run("c", $buffer);
    }

    public function runRepair(&$buffer) {
        return $this->run("r", $buffer);
    }

    //! runs the repair permissions utility
    /*!
     * @param    string    the action for the utility
     * @param    string    the buffer for the results
     *
     * @returns  boolean   whether the utility was executed successfully
     */
    protected function run($action, &$buffer) {
        // list of accepted actions
        $accepted_actions = array("c", "r");

        // sanity check
        if (empty($action) || !in_array($action, $accepted_actions)) {
            return FALSE;
        }

        // build the list of arguments
        $args = array(
            REPAIR_USER_PERMS,
        );

        // only need to append on the repair flag as script runs in check mode by default
        if ($action === "r") {
            $args[] = "-$action";
        }

        // execute the repair utility with the secure wrapper
        $rv = $this->execRead($args, $buffer);

        // ensure that the execution was successfuly
        if ($rv !== 0 || !strlen($buffer)) {
            return FALSE;
        }

        return TRUE;
    }
}

