<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id:
 */
include_once('lib.cmdwrapper.php');

class ReplayUtility extends cmdWrapper {
    // internal protected singleton.
    protected static $object = NULL;
    protected $result = NULL;

    //! static function to get the current singleton instance
    /*!
     * @returns ReplayUtlity  the active singleton instance
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new self(REPLAYER);
        }
        return self::$object;
    }

    //! runs magicspam replayer and stores the result
    /*!
     * @param    array     the mail session information
     * 
     * @returns  boolean   whether the call to the replay utility was successful
     */
    public function callReplayUtility($data) {
        // sanity check
        if (!isset($data) || !is_array($data)) {
            $this->lastError = "The row is empty";
            return FALSE;
        }
        
        // the list of expected arguments
        $expectedArguments = array(
            'ip',
            'rdns',
            'helo',
            'from',
            'mua',
            'rcpt',
        );

        // ensure the expected arguments are sane and safe
        foreach ($expectedArguments as $arg) {
            // sanity check
            if (empty($data[$arg])) {
                $this->lastError = "Missing the expected argument";
                return FALSE;
            }

            // safety check, protect ourselves against null byte injections
            $data[$arg] = str_replace('/0', '', $data[$arg]);
        }

        // the replay utility expects the following arguments
        //   Format: <state> <ip> <rdns> <helo> <from> <rcpt name> <rcpt domain> <mua> <auth user>
        // lets start by splitting the rcpt address into its name and domain
        list($rcpt_name, $rcpt_domain) = explode("@", $data['rcpt'], 2);
        if (empty($rcpt_name) || empty($rcpt_domain)) {
            $this->lastError = "RCPT is not a valid email address";
            return FALSE;
        }

        // now lets build the list of arguments
        // the state and auth id is not important to the replayer, so preset values will be used
        $args = array(
            'M',
            $data['ip'],
            $data['rdns'],
            $data['helo'],
            $data['from'],
            $rcpt_name,
            $rcpt_domain,
            $data['mua'] === 'yes' ? 1 : 0,
            'danielogan'
        );

        // execute the replay utility with the arguments
        // the replay utility is expected to output a json encoded string
        $buffer = '';
        $rv = $this->execRead($args, $buffer);

        // ensure that the execution was successfuly and we got the string
        if ($rv !== 0 || !strlen($buffer)) {
            $this->lastError = "Failed to retrieve replay results";
            return FALSE;
        }

        // decode the json string into an associative array and store
        $this->result = json_decode($buffer, TRUE);
        if (empty($this->result) || count($this->result) !== 2) {
            $this->lastError = "Failed to decode replay results";
            return FALSE;
        }

        // the replay utility has been called successfully
        return TRUE;
    }

    //! retrieves the result of the replay utility
    public function getResult() {
        return $this->result;
    }
}

