<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id:
 */
include_once('lib.cmdwrapper.php');

class SpamAssassinManagerUtility extends cmdWrapper {
    // internal protected singleton.
    protected static $object = NULL;

    //! static function to get the current singleton instance
    /*!
     * @returns SpamAssassinManagerUtility   the active singleton instance
     */
    static public function getInstance() {
        // call the SpamAssassin Manager via the secure wrapper
        // unless the suite is MSPRO WHM
        if (is_null(self::$object)) {
            self::$object = new self(SECURE_BIN);
        }
        return self::$object;
    }

    //! runs the MagicSpam SpamAssassin manager utility
    /*!
     * @param    numeric   the action for the manager
     * @param    string    the buffer for the results
     *
     * @returns  boolean   whether the call to the SA stats utility was successful
     */
    public function callSpamAssassinManagerUtility($action, &$buffer) {
        $accepted_action = array("s", "e", "d");

        // sanity check
        if (empty($action) || !in_array($action, $accepted_action, TRUE)) {
            $this->lastError = "Invalid arguments";
            return FALSE;
        }

        // protect ourselves against null byte injections
        str_replace("\0", "", $action);

        // build the list of arguments
        // NOTE: pass SpamAssassin Manager as an argument since getting
        //       wrapped by MagicSpam Secure
        $args = array(
            SA_MANAGER,
            '-' . $action,
        );

        // execute the SA manager utility with the arguments
        // the SA stats utility is expected to output a json encoded string
        $rv = $this->execRead($args, $buffer);

        // ensure that the execution was successfuly and we got the string
        if ($rv !== 0 || !strlen($buffer)) {
            $this->lastError = "Failed to retrieve SA manager results";
            return FALSE;
        }

        // the SA manager utility has been called successfully
        return TRUE;
    }
}

