<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2010-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Anonymous
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: class.LMNotificationsLocalRepository.php 1174 2016-04-28 17:29:37Z shaun $
 */

require_once('class.LMNotificationsRepository.php');
require_once('class.LMNotificationsEntry.php');

class LMNotificationsLocalRepository extends LMNotificationsRepository {
    //! string. the file suffix for the file we look for.
    private $_file_ext = '.cache';

    //! string. the path to the directory containing the file we want.
    private $_data_dir;

    //! constructor.
    /*!
     * @param string $dataDir Directory containing the feed file.
     * @param integer $product Feed ID.
     * @param string $suffix The file suffix we will look for.
     *
     * @constructor
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    public function __construct($dataDir, $product, $suffix) {
        // TODO: parameter checks.
        parent::__construct($product);
        $this->_data_dir = $dataDir;
        $this->_file_ext = $suffix;
    }

    public function get($number, $offset = 0) {
        $this->load(0);

        $entries = $this->getEntries();
        $entries = array_slice($entries, $offset, $number, TRUE);
        // sort by key, descending.
        krsort($entries, SORT_NUMERIC);
        reset($entries);
        return $entries;
    }

    //! replace the set of LMNotificationEntry objects internally.
    /*!
     * @param array $entries Associative array of timestamp
     *   => LMNotificationEntry objects. This will replace the one
     *   we have internally.
     *
     * @return void
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * TODO: should we really enforce this format of associative array? it's
     *   a little non-obvious. perhaps instead we could just take a regular
     *   numeric indexed array (0, 1, etc).
     */
    public function setEntries($entries) {
        // TODO: parameter check?
        $this->entries = $entries;
    }

    public function addEntries($entries) {
        foreach ($entries as $entry) {
            // TODO: use addEntry()?
            $this->entries[$entry->timestamp] = $entry;
        }
    }

    //! load entries since the given unixtime.
    /*!
     * @param numeric $timestamp  Unixtime to load entries since
     * @param array   $alwaysLoad Optional array of entry 'types' that should always be loaded
     *
     * @return void
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    protected function load($timestamp, $alwaysLoad = NULL) {
        $file = $this->_data_dir . DIRECTORY_SEPARATOR . $this->product
            . $this->_file_ext;
        if (!file_exists($file)) {
            // this is not an error.
            return;
        }
        if (!is_readable($file)) {
            // TODO: we should probably log an error here
            return;
        }
        $contents = @file_get_contents($file);
        if ($contents === FALSE) {
            // TODO: should probably log something here too
            return;
        }
        $entries = @unserialize($contents);
        if ($entries === FALSE) {
            // TODO: here too.. corrupt entries?
            return;
        }
        if (!is_array($entries)) {
            // TODO: ..
            return;
        }
        // now that we have our entries, add them to the list
        foreach ($entries as $entry) {
            // if we have been passed entry types to always load
            if (is_array($alwaysLoad)) {
                // and the entry type matches the criteria
                if (in_array($entry->type, $alwaysLoad)) {
                    // then add it and go to next entry
                    $this->addEntry($entry);
                    continue;
                }
            }
            // fall through to test based on timestamp criteria
            if ($entry->timestamp > $timestamp) {
                $this->addEntry($entry);
            }
        }
    }

    //! save entries in this object to disk.
    /*!
     * @return bool Whether successful
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * TODO: where do we deal with the behaviour of only saving if a change
     *   occurred?
     */
    public function save() {
        $file = $this->_data_dir . DIRECTORY_SEPARATOR . $this->product
            . $this->_file_ext;
        if (file_exists($file) && !is_writeable($file)) {
            // TODO: log why?
            return FALSE;
        }
        if (!is_writable(dirname($file))) {
            // TODO: log why?
            return FALSE;
        }
        if (file_put_contents($file, serialize($this->entries)) === FALSE) {
            // TODO: log why?
            return FALSE;
        }
        return TRUE;
    }
}
