<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Lucas Ninicao <lucasn@linuxmagic.com>
 *
 * $Id: countryStatsModule.php 10940 2020-01-14 00:44:10Z deiony $
 */

require_once('lib.module.php');

class countryStatsModule extends Module {

    //! class constructor
    public function __construct($l) {
        global $modules;

        parent::__construct($l, FALSE);
        $this->submodules = array('countryEmailStats', 'countryAuthStats');

        // load the submodules
        foreach ($this->submodules as $m) {
            loadModule($m);
        }

        // assign friendly names to submodules
        $countryEmail = $modules['countryEmailStats'];
        $countryAuth = $modules['countryAuthStats'];

        // set the previously posted values of the form not being submitted
        // NOTE: the SESSION stores the previously posted values of the two forms
        if (isset($_POST['top-email']) ) {
            // the email stats is being posted, set the auth stats if necessary
            if (isset($_SESSION['auth_form']['top-auth'],
                $_SESSION['auth_form']['period-auth'])) {
                $countryAuth->setTimePeriod($_SESSION['auth_form']['period-auth']);
                $countryAuth->setTopNum($_SESSION['auth_form']['top-auth']);
            }
        } else if (isset($_POST['top-auth']) ) {
            // the auth stats is being posted, set the email stats if necessary
            if (isset($_SESSION['email_form']['period-email'],
                $_SESSION['email_form']['email-type'],
                $_SESSION['email_form']['top-email'])) {
                $countryEmail->setTimePeriod($_SESSION['email_form']['period-email']);
                $countryEmail->setEmailType($_SESSION['email_form']['email-type']);
                $countryEmail->setTopNum($_SESSION['email_form']['top-email']);
            }
        } else {
            // unset the stored forms values if neither forms are being posted
            unset($_SESSION['auth_form']);
            unset($_SESSION['email_form']);
        }
    }

    //! call child modules and set values posted by the forms
    /*!
     * @param   string          the action being requested
     * @param   array           the array where error messages can be stacked
     * @param   array           the array where any friendly notices can be stacked
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;

        foreach ($this->submodules AS $submodule) {
            if (isset($modules[$submodule])) {
                $modules[$submodule]->actionProcessor($action, $errors, $notices);
            }
        }
    }

    //! method to render the full page module
    public function renderModule() {
        global $modules;

        // render the child modules
        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }
            $modules[$m]->renderModule();
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering
    public function getCSS() {
        return array('today-stats.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering
    public function getJS() {
        return $this->subModuleJS();
    }
}
