<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: exemptionModule.php 11866 2020-07-08 00:08:11Z deiony $
 */
require_once("lib.module.php");

class exemptionModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array (
                                    'userExemption',
                                    'fromWhiteList',
                                    'fromBlackList',
                                    'ipWhiteList',
                                    'ipBlackList',
                                    'ipRLWhiteList',
                                    'userRLWhiteList');

        // professional only exemptions
        if (isProfessional()) {
            // insert the pro exemptions from the 6th position onwards
            array_splice($this->submodules, 5, 0, array('countryBlackList'));
            array_splice($this->submodules, 5, 0, array('ipAuthWhiteList'));
            array_splice($this->submodules, 5, 0, array('userAuthWhiteList'));
        }

        // set module to be migratable
        $this->migratable = TRUE;

        foreach ($this->submodules as $m) {
            loadModule($m);
        }

    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])
           && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module
    /*!
     * this is an override to allow mixed behavior between
     * 'container' and 'regular' module - for working with sub category
     * foldout widgets.
     */
    public function renderModule() {
        global $modules;

        // Prints the blurb describing the purpose of the Exemptions page
        print "<h2>" . intl('exemption') . "</h2>";
        print "<div class=\"masterBlurb\"><p>";
        print intl('exemptionBlurb') . "</p></div>";

        // dictionaries for the js messages
        $this->_i18n_js[] = 'exemptionsConfirmDelete';
        $this->_i18n_js[] = "comment_for";
        $this->_i18n_js[] = "remaining_characters";
        $this->_i18n_js[] = 'invalid_entry';
         
        // After printing out the blurb, the submodule foldouts get generated here
        $submodule = (isset($_POST['submodule']) && strlen($_POST['submodule'])) ? $_POST['submodule'] : NULL;

        // Prints a small div to force all of the drop-downs to have grey backgrounds
        echo "<div class=\"enforceGrey\">";

        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }
            $activeAddon = "";
            $isActive = FALSE;
            $img = 'expand-icon.png';
            // we determine if this tab should be displayed expanded or not.
            // if no submodule was requested, then we don't expand anything
            if (!is_null($submodule)) {
                if ($m == $submodule) {
                    $isActive = TRUE;
                }
            }

            // Assemble the name of the session variable
            $collapseState = $m . "__collapse";

            // Validation to make certain we got a value
            if (isset($_SESSION[$collapseState]) && $_SESSION[$collapseState] != '') {
                $collapseString = $_SESSION[$collapseState];
            } else {
                $collapseString = 0;
            }

            // If 1 then we set it to be active
            if ($collapseString == 1) {
                $isActive = TRUE;
            }

            // Define the session variable here
            $_SESSION[$collapseState] = $collapseString;

            // Alters the style based if the tab is active
            if ($isActive) {
                $activeAddon = " collapsibleModuleActive";
                $img = 'collapse-icon.png';
                $firstExpanded = TRUE;
            }

            // Sets the entry count and the basic data
            $modules[$m]->setEntryData();

            // Outputs a collapsible
            echo "<div class=\"collapseHeader\" onclick=\"toggleSectionAndUpdateState(this, '{$m}','{$collapseString}','" . IMAGEPATH . "');\">\n";
            echo "  <div class=\"collapseLeft\"></div>\n";
            echo "  <div class=\"collapseCenter\">\n";
            echo "    <span class=\"collapseTitle\">" . $modules[$m]->getModuleTitle() . " (";
            echo $modules[$m]->getEntryCount() . ")</span>\n";
            echo "    <span class=\"collapseActions\"><img src=\"" . IMAGEPATH . "{$img}\"/></span>\n";
            echo "  </div>\n";
            echo "  <div class=\"collapseRight\"></div>\n";
            echo "</div><!-- end collapseHeader-->\n";
            echo "<div id=\"{$m}\" class=\"collapsibleModule{$activeAddon}\">\n";

            $modules[$m]->renderModule();

            echo "</div><!-- end collapsibleModule -->\n";
        }

        // modal for adding comments to an entry
        echo "<div id='exemptionCommentModal'>";
        echo "<div id='exemptionCommentContainer' class='screenCenter'>";
        echo "  <strong id='exemptionCommentTitle'>" . intl("comment_for") . "</strong>";
        echo "  <span id='exemptionCommentCloseButton'>X</span>";
        echo "  <p id='exemptionCommentEntry'></p>";
        echo "  <textarea id='exemptionCommentInput' maxlength='255'></textarea>";
        echo "  <div id='exemptionCommentButtonRow'>";
        echo "    <span id='exemptionCommentRemainder'>" . intl("remaining_characters") . ": 255</span>";
        echo "    <button id='exemptionCommentDeleteButton' class='msbutton' type='button'>";
        echo "      <span>" . intl("delete") . "</span>";
        echo "      </button>";
        echo "    <button id='exemptionCommentSaveButton' class='msbutton' type='button'>";
        echo "      <span>" . intl("save") . "</span>";
        echo "    </button>";
        echo "  </div>";
        echo "</div>";
        echo "</div>";

        echo "</div><!-- end enforceGrey -->";
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array_merge(array('exemption.css'), $this->subModuleCSS());
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array_merge(array('form-submit.js', 'ajax.js'), $this->subModuleJS());
    }
}
