<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: feDashboardModule.php 4785 2015-03-13 20:42:37Z shaun $
 */
require_once('lib.module.php');

class feDashboardModule extends Module {

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);
        // this page contains two widgets: todays statistics and domains.
        $this->submodules = array('feGlance', 'monthStats', 'userDomains');
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }
    public function getPageTitle() {
        return intl($this->label);
    }
    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        // XXX: consider an abstracted method in parent to handle submodule passoff.
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ( $subModule && isset($modules[$subModule]) && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
            // reload the submodules after updating the active domains as it may
            // affect the data in the glance module
            if ($_POST['submodule'] == 'userDomains' && $_POST['ms_action'] == 'updateActiveDomains') {
                foreach ($this->submodules as $m) {
                    loadModule($m);
                }
            }
        }
    }

    //! method to render the full page module.
    public function renderModule() {
        global $modules;
        renderMenu();
        
        foreach ($this->submodules as $idx => $m) {
            if (isset($modules[$m])) {
                echo "<div class=\"subModule\">\n";
                $modules[$m]->renderModule();
                echo "</div><!-- end subModule " . htmlspecialchars($m);
                echo " -->\n";
            }
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return $this->subModuleJS();
    }
}

?>
