<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Kyle Macdonald <kylem@linuxmagic.com>
 *            Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: feIpReputationModule.php 4785 2015-03-13 20:42:37Z shaun $
 */
require_once("lib.module.php");

class feIpReputationModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);
        $this->submodules = array (
            'bmsLists',
        );
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }
    public function getPageTitle() {
        return intl($this->label);
    }
    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }


    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array_merge(array('form-submit.js','help-popup.js','ajax.js'), $this->subModuleJS());
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Kyle Macdonald <kylem@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;

        // Go ahead and generate the menu
        renderMenu();
       
        // If the module isn't found, return
        if (!isset($modules['bmsLists'])) {
            return;
        }

        // XXX: inline echo is EVIL
        echo "<div class=\"collapseHeader collapseHeaderAlwaysOpen\">\n";
        echo "  <div class=\"collapseLeft\"></div>\n";
        echo "  <div class=\"collapseCenter\">\n";
        echo "    <span class=\"collapseTitle\">" . $modules['bmsLists']->getModuleTitle() . "</span>\n";
        echo "  </div>\n";
        echo "  <div class=\"collapseRight\"></div>\n";
        echo "</div><!-- end collapseHeader-->\n";
        echo "<div id=\"bmsLists\" class=\"collapsibleModule collapsibleModuleActive\">\n";

        $modules['bmsLists']->renderModule();

        echo "</div><!-- end collapsibleModule -->\n";
    }
}
