<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *            Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: monthStatsModule.php 4617 2014-12-17 18:51:59Z shaun $
 */
require_once('lib.module.php');
require_once('lib.hourlystats.php');

class hourlyStatsModule extends Module {

    //! internal flag for Microsoft Management Console (MailEnable Admin Console)
    protected $isMMC;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->isMMC = ($_SERVER['SERVER_PORT'] == 6242);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        $stats = new HourlyStats();

        //Prepare labels
        $header = intl('hourlyStatsHeader');


        // and load up the translated year/month stuff from there.

        // add the i18n we need in javascript.
        $this->_i18n_js[] = 'hourlyStatsCloseLabel';
        $this->_i18n_js[] = 'hourlyStatsHelp';

        renderTemplate('hourlyStats', array(
            'module'               => $module,
            'stats'                => &$stats,
            'header'               => $header,
            'target'               => $this->isMMC ? "" : "logSearchHour",
            'hourlyStatsBlurb'     => intl('hourlyStatsBlurb'),
            'hourlyStatsHelpLabel' => intl('hourlyStatsHelpLabel')
        ));

    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function getCSS() {
        return array('hourly-stats.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function getJS() {
        return array('month-stats.js', 'help-popup.js', 'hour-stats.js');
    }

}
