<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2020-2021 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: ipAuthModule.php 12127 2021-02-01 23:01:44Z deiony $
 */

require_once('functions.php');
require_once('lib.module.php');

class ipAuthModule extends Module {

    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
    }

    //! process any actions
    /*!
     * @param   string  the action being requested
     * @param   array   the array where error messages can be stacked
     * @param   array   the array where any 'friendly' notices can be stacked
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if (isset($action) && WV::isStringNotEmpty($action) && $action == "save") {
            $block_auth_azure = $_POST["block_auth_azure"] === "true" ? 1 : 0;
            $block_auth_amazon = $_POST["block_auth_amazon"] === "true" ? 1 : 0;
            $block_auth_google = $_POST["block_auth_google"] === "true" ? 1 : 0;
            $block_auth_rats = $_POST["block_auth_rats"] === "true" ? 1 : 0;
            if (setControl('block_auth_azure', $block_auth_azure) !== TRUE
                    || setControl('block_auth_amazon', $block_auth_amazon) !== TRUE
                    || setControl('block_auth_google', $block_auth_google) !== TRUE
                    || setControl('block_auth_rats', $block_auth_rats) !== TRUE) {
                $errors[] = intl('ip_auth_save_fail');
                return;
            }
            $notices[] = intl('ip_auth_save_success');
        }
    }

    //! render the module
    public function renderModule() {
        global $module;

        // retrieve the ip auth restrictions
        $block_auth_azure = getControl("block_auth_azure", 0) == 1;
        $block_auth_amazon = getControl("block_auth_amazon", 0) == 1;
        $block_auth_google = getControl("block_auth_google", 0) == 1;
        $block_auth_rats = getControl("block_auth_rats", 0) == 1;

        // pass i18n to frontend for help popups
        $this->_i18n_js[] = 'block_auth_azure_help';
        $this->_i18n_js[] = 'block_auth_amazon_help';
        $this->_i18n_js[] = 'block_auth_google_help';
        $this->_i18n_js[] = 'block_auth_rats_help';

        // render the module via template
        renderTemplate('ipAuth', array(
            "module" => $module,
            "block_auth_azure" => $block_auth_azure,
            "block_auth_amazon" => $block_auth_amazon,
            "block_auth_google" => $block_auth_google,
            "block_auth_rats" => $block_auth_rats,
        ));
    }

    //! retrieve any optional CSS files for rendering
    public function getCSS() {
        return array('ip-auth.css');
    }

    //! retrieve any optional JS files for rendering
    public function getJS() {
        return array('help-popup.js', 'form-submit.js');
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // the list of options
        $options = array(
            "block_auth_azure",
            "block_auth_amazon",
            "block_auth_google",
            "block_auth_rats",
        );

        // retrieve and store the value of each option
        foreach ($options as $option) {
            $configurations[$option] = getControl($option, 0);
        }

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // load each option with the value
        foreach ($configurations as $option => $value) {
            setControl($option, $value);
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
