<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: ipRateLimiterModule.php 4221 2014-11-04 23:28:57Z shaun $
 */
require_once('lib.module.php');

class ipRateLimiterModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array(
                                  'ipRateLimitList'
                                 );
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])
           && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module.
    /*!
     * @author        Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;
        foreach ($this->submodules as $idx => $m) {
            if (isset($modules[$m])) {
                echo "<div class=\"subModule\">\n";
                $modules[$m]->renderModule();
                echo "</div><!-- end subModule " . htmlspecialchars($m);
                echo " -->\n";
            }
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return $this->subModuleJS();
    }
}
