<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: bmsListsModule.php 6285 2017-03-30 17:24:13Z shaun $
 */
require_once('lib.module.php');
require_once('lib.datatable.php');

class ipReputationTemplateModule extends Module {
    //! internal array of all available BMS lists.
    private $lists;
    //! expected array label used for rendered radio buttons
    private $radio_name;

    // constants for the status of settings
    const SETTING_STATUS_OFF = 0;
    const SETTING_STATUS_FLAG = 1;
    const SETTING_STATUS_ON = 2;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);

        // set module to be migratable
        $this->migratable = TRUE;

        // XXX: consider extending bmsListsModule instead of redefining
        //      essentially the same information
        $this->lists = array(
            '4' => array(
                    'description' => 'UCEPROTECT-1',
                    'default' => self::SETTING_STATUS_OFF,
                    'help' => 'bmsUCE1'),
            '5' => array(
                    'description' => 'UCEPROTECT-2',
                    'default' => self::SETTING_STATUS_OFF,
                    'help' => 'bmsUCE2'),
            '13' => array(
                    'description' => 'PSBL',
                    'default' => self::SETTING_STATUS_FLAG,
                    'help' => 'bmsPSBL'),
            '23' => array(
                    'description' => 'SORBS-DUL',
                    'default' => self::SETTING_STATUS_OFF,
                    'help' => 'bmsSORBS'),
            '35' => array(
                    'description' => 'MIPSpace-all (includes MIPSpace-worst, poor, and pros)',
                    'default' => self::SETTING_STATUS_OFF,
                    'help' => 'bmsMIPSall'),
            '40' => array(
                    'description' => 'MIPSpace-worst',
                    'default' => self::SETTING_STATUS_ON,
                    'help' => 'bmsMIPSworst'),
            '41' => array(
                    'description' => 'MIPSpace-poor',
                    'default' => self::SETTING_STATUS_FLAG,
                    'help' => 'bmsMIPSpoor'),
            '42' => array(
                    'description' => 'MIPSpace-pros',
                    'default' => self::SETTING_STATUS_OFF,
                    'help' => 'bmsMIPSpros'),
            '36' => array(
                    'description' => 'RATS-Dyna',
                    'default' => self::SETTING_STATUS_ON,
                    'help' => 'bmsRatsDyna'),
            '37' => array(
                    'description' => 'RATS-NOPTR',
                    'default' => self::SETTING_STATUS_ON,
                    'help' => 'bmsRatsNoptr'),
            '38' => array(
                    'description' => 'RATS-Spam',
                    'default' => self::SETTING_STATUS_ON,
                    'help' => 'bmsRatsSpam'),
        );
        $this->radio_name = "bmslist";
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // init variables
        $selected_lists = "";
        $selected_arr = array();
        $flagarr = array();
        $ret = TRUE;

       	if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }	

        if ($action == 'save_lists') {
            if ($this->handleBmsListSave() === FALSE) {
                $errors[] = intl('errorFailSave');
            }

        } else if ($action == "set_bmslist_to_defaults") {
            $bms_on = array();
            $bms_flag = array();

            foreach ($this->lists as $bms => $props) {
                switch ($props['default']) {
                case self::SETTING_STATUS_FLAG:
                    $bms_flag[] = $bms;
                case self::SETTING_STATUS_ON:
                    $bms_on[] = $bms;
                    break;
                case self::SETTING_STATUS_OFF:
                default:
                    break;
                }
            }

            $ret = setControl(BMS_TEMPLATE_ENABLED, implode("\n", $bms_on));
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $ret = setControl(BMS_TEMPLATE_FLAG, implode("\n", $bms_flag));
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $notices[] = intl("bmsLists"). " " . intl("saved");

        } else if ($action == "sync_bms_list") {
            // ensure that there is a bms list to sync
            if (empty($_POST['bms-num'])) {
                $errors[] = intl('listSyncError');
                return;
            }
            $bms_list = $_POST['bms-num'];

            // ensure that the individual bms list has been saved
            if ($_POST['ori_bmslist'][$bms_list] !== $_POST['bmslist'][$bms_list]) {
                if ($this->handleBmsListSave() === FALSE) {
                    $errors[] = intl('errorFailSave');
                    return;
                }
            }

            // sync the individual bms list
            if ($this->handleBmsListSync($bms_list) === FALSE) {
                $errors[] = intl('listSyncError');
                return;
            }

            $notices[] = $_POST['bms-num'] . ' ' . intl('policySynced');
        } else if ($action == "sync_all_bms_list") {
            // ensure that all the bms lists have been saved
            foreach ($_POST['ori_bmslist'] as $bms_list => $status) {
                if ($status !== $_POST['bmslist'][$bms_list]) {
                    if ($this->handleBmsListSave() === FALSE) {
                        $errors[] = intl('errorFailSave');
                        return;
                    }
                    break;
                }
            }

            // sync all the bms lists
            if ($this->handleBmsListSyncAll() === FALSE) {
                $errors[] = intl('errorFailSyncAll');
                return;
            }

            $notices[] = intl('noticeSuccessSyncAll');
        }

    }

    //! Save all the BMS lists into template files
    protected function handleBmsListSave() {
        // create two empty arrays for keeping track of enabled and flagged bms
        $enabledBms = array();
        $flaggedBms = array();

        // sanity check
        if (empty($_POST["bmslist"])) {
            return FALSE;
        }

        // keep track of which bms have been enabled or flagged
        foreach ($_POST["bmslist"] as $bms=>$status) {
            // add rule to corresponding array
            if ($status !== "false") {
                $enabledBms[] = $bms;
                if ($status === "flag") {
                    $flaggedBms[] = $bms;
                }
            }
        }

        // regardless of how the bms are set, we always remove all
        // the bms from both the enabled and flagged files
        // start removal from the enabled file
        $ret = clearControl(BMS_TEMPLATE_ENABLED);
        if ($ret !== TRUE) {
            return FALSE;
        }
        // continue on removing from the flagged file
        $ret = clearControl(BMS_TEMPLATE_FLAG);
        if ($ret !== TRUE) {
            return FALSE;
        }

        // at this point, both the enabled and flagged files will be empty
        // so we can add bms to them as necessary and not worry about any
        // more removals

        // start by creating a single string for both enabled and flagged bms
        $enabledString = implode("\n", $enabledBms);           
        $flaggedString = implode("\n", $flaggedBms);           

        // then add enabled bms to the enabled file
        if (strlen($enabledString)) {
            $ret = setControl(BMS_TEMPLATE_ENABLED, $enabledString);
            if ($ret !== TRUE) {
                return FALSE;
            }
        }

        // finally add the flagged bms to the flagged file
        if (strlen($flaggedString) > 0) {
            $ret = setControl(BMS_TEMPLATE_FLAG, $flaggedString);
            if ($ret !== TRUE) {
                return FALSE;
            }
        }

        return TRUE;
    }

    protected function handleBmsListSyncAll() {
        // sanity check
        if (!is_array($_POST['bmslist'])) {
            return false;
        }

        // go through and sync each spam setting
        foreach ($_POST['bmslist'] as $bms => $key) {
            if ($this->handleBmsListSync($bms) === FALSE) {
                return false;
            }
        }

        return true;
    }

    /**
     * Handles the global syncing of a BMS list. Calls a CLI program.
     *
     * @param array $errors the list of errors
     * @param array $noticies the list notices
     *
     */
    protected function handleBmsListSync($bms_list) {
        // sanity check
        if (empty($_POST[$this->radio_name][$bms_list])) {
            return false;
        }

        $listSliderOption = $_POST[$this->radio_name][$bms_list];
        if ($listSliderOption === 'true') {
            $cmdOption = GlobalToggle::ENABLE;
        } else if ($listSliderOption === 'false') {
            $cmdOption = GlobalToggle::DISABLE;
        } else if ($listSliderOption === 'flag') {
            $cmdOption = GlobalToggle::FLAG;
        } else {
            return false;
        }

        $toggleCmd = GlobalToggle::getInstance();
        if($toggleCmd->toggleIPRepList($bms_list, $cmdOption) === FALSE) {
            return false;
        }

        return true;
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        // add all the bms help labels for the js
        foreach ($this->lists as $rule=>&$data) {
            $this->_i18n_js[] = $data['help'];
        }

        // set up the table style
        $table = new magicspamDataTable();
        $table->setClass("row", array("leftColumn", "centerColumn", "rightColumn", "pushBtn"));
        $table->setClass("headerRow", array("leftColumn", "centerColumn", "rightColumn", "pushBtn"));

        // create two empty arrays for retrieving the enabled and
        // flagged bms from their respective template files
        $lists_enabled = array();
        $lists_flag = array();

        // start by retrieving the ena
        $ret = getControl(BMS_TEMPLATE_ENABLED, "");
        if (strlen($ret) > 0) { // something was returned, set the rules and do the count
            $lists_enabled = explode("\n", $ret);
        }

        $ret = getControl(BMS_TEMPLATE_FLAG, "");
        if (strlen($ret) > 0) { // something was returned, set the rules and do the count
            $lists_flag = explode("\n", $ret);
        }

        $listlabel = intl('bmsListLabel');
        foreach ($this->lists as $rule => &$data) {
            $help = isset($data['help']) ? $data['help'] : "";
            $desc = htmlspecialchars(isset($data['description']) ? $data['description'] : "");

            $internal = "({$listlabel}: " . htmlspecialchars($rule) . ")";
            $target = "_blank";
            $desc = renderListDescription($desc, $internal, NULL, "MS.showHelpPopup(this,'X','".$help."'); return false;", $target);

            $isEnabled = FALSE;
            if (in_array($rule, $lists_enabled)) {
                $isEnabled = TRUE;
                if (in_array($rule, $lists_flag)) {
                    $isEnabled = 'flag';
                }
            }

            $defstat = generateLabel("recommended", $data['default'] !== self::SETTING_STATUS_OFF ? TRUE : FALSE);
            if ($data['default'] === self::SETTING_STATUS_FLAG) {
                $defstat = str_replace("isdefault", "isdefaultflag", $defstat);
            }
            
            $sliderWidget = renderSliderThree($isEnabled, 0, $this->radio_name . "[{$rule}]", 'bmsListsForm');

            $btn = '<button class="msbutton" type="button" data-number="' . $rule . '"onclick="syncBmsList(event)" ><span>' . intl('sync') . '</span></button>';
            $table->addRow(array($desc, $defstat, $sliderWidget, $btn));
        }

        if (isset($_SESSION['ipReputationTemplate__collapse']) && $_SESSION['ipReputationTemplate__collapse'] != '') {
            $collapseState = $_SESSION['ipReputationTemplate__collapse'];
        
        // Else open by default
        } else {
            $collapseState = 1;
        }

        $header = '<span class="collapseTitle">' . intl('bmsLists') . ' Template' . "</span>\n"
                . '<span class="msTitleFont listStatus">' . intl('status') . "</span>\n"
                . '<span class="msTitleFont listAction">' . intl('sync') . "</span>\n";

        $this->_i18n_js[] = 'on';
        $this->_i18n_js[] = 'off';

        $this->_i18n_js[] = 'ipAddressInvalid';

        $blurb = intl('bmsBlurb');

        renderTemplate('bmsListsTemplate', array(
            'header'        => $header,
            'table'         => $table->getRenderString(),
            'module'        => $module,
            'collapseState' => $collapseState,
        ));
    }

    //! method to retrieve the title of the current module
    /*!
     * @returns string  the title for the current module.
     */
    public function getModuleTitle() {
        return intl('bmsLists');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('bms.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array("set-action.js", "bms-lists.js");
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // retrieve and store the enabled and flag only BMS lists templates
        $configurations[BMS_TEMPLATE_ENABLED] = getControl(BMS_TEMPLATE_ENABLED, "", MASTERCONTROLDIR);
        $configurations[BMS_TEMPLATE_FLAG] = getControl(BMS_TEMPLATE_FLAG, "", MASTERCONTROLDIR);

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // store the enabled BMS lists template
        if (array_key_exists(BMS_TEMPLATE_ENABLED, $configurations)
            && WV::isStringNotEmpty($configurations[BMS_TEMPLATE_ENABLED])) {
            setControl(BMS_TEMPLATE_ENABLED, $configurations[BMS_TEMPLATE_ENABLED]);
        } else {
            clearControl(BMS_TEMPLATE_ENABLED);
        }

        // store the flag only BMS lists template
        if (array_key_exists(BMS_TEMPLATE_FLAG, $configurations)
            && WV::isStringNotEmpty($configurations[BMS_TEMPLATE_FLAG])) {
            setControl(BMS_TEMPLATE_FLAG, $configurations[BMS_TEMPLATE_FLAG]);
        } else {
            clearControl(BMS_TEMPLATE_FLAG);
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
