<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: logSearchModule.php 8862 2018-11-27 19:11:35Z deiony $
 */
require_once('lib.module.php');
require_once('wizard/lib.wizard_validator.php');

class logTableModule extends Module
{
    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    public function actionProcessor($action, &$errors, &$notices) {
        // no actions to process
    }

    //! renders the display for screen, this is an overwritten method
    /*!
     * @returns void
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     * @author Robert Bisewski <robertb@linuxmagic.com>
     *
     */
    public function renderModule()  {
        renderTemplate('logTable', array(
            "logTable" => $this->makeLogTable(),
        ));
    }

    //! method to retrieve any 'additional' JS files to include in the page rendering
    /*!
     * @returns array   an array of the files to include
     */
    public function getJS() {
        return array(
            'help-popup.js',
            'magicspam.js',
            'magicspam-menu.js',
            'limit-integer.js',
            'check-enabled.js',
            'ajax.js',
            'form-submit.js',
            'log-search.js',
        );
    }

    public function getCSS() {
        return array();
    }

    //! method to build log summary table with 2 sub-tables
    /*!
     * @return a string with html ready to be printed as a table
     *
     * @author Daniel Peitl <danielp@linuxmagic.com>
     */
    protected function makeLogTable() {
        // Summary of logs and stats files sizes
        // Note for filesize() usage: Because PHP's integer type is signed and many platforms use 32bit integers,
        // some filesystem functions may return unexpected results for files which are larger than 2GB.
        $sizeStat = 0; $sizeLog = 0; $sumStat = 0; $sumLog = 0;
        $largestFile = 0; $largestFileName = ""; $lastFile = 0; $lastFileName = "";
        $retLogTable = "";

        $logDir = LOGDIR;
        if (is_dir($logDir)) {
            $itemHandler = @opendir($logDir);
            if ($itemHandler === FALSE) {
                // TODO: raise an error?
                return '';
            }
            while (($item = readdir($itemHandler)) !== FALSE) {
                // it's a file of interest?
                if (preg_match("/^mslog/", $item)) {
                    $fileDate = @filectime($logDir.$item);
                    if ($fileDate === FALSE) {
                        // as a fallback... we could also raise error...
                        $fileDate = 0;
                    }
                    $fileSize = @filesize($logDir.$item);
                    if ($fileSize === FALSE) {
                        // as a fallback... we could also raise error...
                        $fileSize = 0;
                    }
                    $sizeLog += $fileSize;
                    $sumLog++;
                } else if (preg_match("/^\.counter/", $item)) {
                    $fileDate = @filectime($logDir.$item);
                    if ($fileDate === FALSE) {
                        // as a fallback... we could also raise error...
                        $fileDate = 0;
                    }
                    $fileSize = @filesize($logDir.$item);
                    if ($fileSize === FALSE) {
                        // as a fallback... we could also raise error...
                        $fileSize = 0;
                    }
                    $sizeStat += $fileSize;
                    $sumStat++;
                } else {
                    continue;
                }

                // it's the largest?
                if ($fileSize !== FALSE && $fileSize > $largestFile) {
                    $largestFile = $fileSize;
                    $largestFileName = $item;
                }
                // it's the most recent?
                if ($fileDate !== FALSE && $fileDate > $lastFile) {
                    $lastFile = $fileDate;
                    $lastFileName = $item;
                }
            }
            closedir($itemHandler);
        }

        $totalSize = $this->formatFilesize($sizeLog + $sizeStat)." (".$this->formatFileSize($sizeLog)." / ".$this->formatFileSize($sizeStat).")";
        $largestFile = htmlspecialchars($largestFileName)." (".$this->formatFilesize($largestFile).")";

        $logDays = htmlspecialchars(getControl("logdays"));
        if (!isset($logDays) || !strlen($logDays)) {
            $logDays = 7;
        }
        $logDays .= intl('lsx_days');
        $logDays .= '<a class="logDaysSettings" href="?module=settings" '
                  . 'onclick="doFormSubmit({\'hiddenFields\':{\'module\''
                  . ':\'settings\',\'page\':\'system_settings\'}}); '
                  . 'return false;" title="'
                  . intl('lsx_settings') . '">'
                  . intl('lsx_settings') . "</a>";

        if ($lastFile > 0) {
            $lastFile = htmlspecialchars(date("l, F jS g:i a", $lastFile));
        } else {
            $lastFile = intl('lsx_noentries');
            $largestFile = intl('lsx_noentries');
            $totalSize = intl('lsx_noentries');
        }

        $logTableLeft = new magicspamDataTable;
        $logTableLeft->addRow(array(intl('lsx_loglocation'), htmlspecialchars($logDir)));
        $logTableLeft->addRow(array(intl('lsx_retention'), $logDays));
        // NOTE: lastFile is already html encoded!
        $logTableLeft->addRow(array(intl('lsx_last'), $lastFile));

        $logTableRight = new magicspamDataTable;
        $logTableRight->addRow(array(intl('lsx_totalfiles'), $sumLog+$sumStat));
        $logTableRight->addRow(array(intl('lsx_totalsize'), $totalSize));
        $logTableRight->addRow(array(intl('lsx_largest'), $largestFile));

        $logTable = "<div id='logStats'>". $logTableLeft->getRenderString() ."</div><!-- end #logStats -->"
                  . "<div id='logInfo'>". $logTableRight->getRenderString() ."</div><!-- end #logInfo -->";

        return $logTable;
    }

    //! method to format filesize to be displayed with proper unit
    /*!
     * @param   integer $size in bytes
     *
     * @return  mixed string size with unit or blank string if you pass
     *   in something invalid.
     *
     * @author Daniel Peitl <danielp@linuxmagic.com>
     *
     * Adapted from: http://www.php.net/manual/en/function.filesize.php
     */
    protected function formatFilesize($size) {
        // Basic validation
        if (!is_numeric($size) || $size < 0) {
            return '';
        }
        // Kibibyte units
        $units = array('B', 'KiB', 'MiB', 'GiB', 'TiB');
        // Reduces the number until it gets a proper size
        // each iteration changes the index of $units to be used
        $mod = 1024;
        for ($i = 0; $size > $mod && $i < count($units); $i++) {
            $size /= $mod;
        }
        // outputs the file size and the proper unit
        return round($size, 2)." ".$units[$i];
    }
}
