<?php

/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * This is the primary working interface page for the WHM module.
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id$
 */

require_once('lib.module.php');

class logoutModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if (is_string($action) && $action === 'logout') {
            session_unset();
            session_destroy();
            // and now we force a redirect to /
            header('Location: /');
            exit(0);
        }
    }

    //! method to render the full page module.
    /*
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;
        renderMenu();
        renderTemplate('logout',
                       array(
                            'confirmation' => intl('lo_confirmation')
                           ,'module' => $module
                           ,'logout' => intl('logout')
                           ));
    }

    public function getPageTitle() {
        return 'logout';
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('logout.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array();
    }
}

