<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Sean West <sean@linuxmagic.com>
 *            Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: monthStatsModule.php 10940 2020-01-14 00:44:10Z deiony $
 */
require_once('lib.module.php');
require_once('lib.rcptcounter.php');
require_once('lib.monthstats.php');

class monthStatsModule extends Module {
    private $month, $renderSelect;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // unless we have a request, we default to "this" month.
        $this->month = date('n');
        $this->renderSelect = TRUE;
    }

    //! method to disable the month-selector
    public function setRenderSelect($val) {
        if (is_bool($val)) {
            $this->renderSelect = $val;
        }
    }

    //! method to retrieve month-selection state
    public function getRenderSelect() {
        return $this->renderSelect; 
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // hooks for month selection
        if ($action == 'selectMonth') {
            if (isset($_POST['month'])) {
                $this->month = $_POST['month'];
            }
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        $months = $this->loadMonths();

        //Prepare labels
        $header = intl('monthStatsHeader');
        $hBlurb = intl('monthStatsBlurb');

        // create our statstable object
        $stats = new monthStats($this->month, $months, false);

        // and load up the translated year/month stuff from there.
        $this->month = $stats->getMonth();
        $year = $stats->getYear();

        // TODO: why check both month and year being less than current?
        //   what condition are we trying to alert of? seems odd...
        if ($year < date('Y') && $this->month <= date('n')) {
            $year = "<span class=\"alertYear\">{$year}</span>";
        }

        // add the i18n we need in javascript.
        $this->_i18n_js[] = 'monthStatsCloseLabel';
        $this->_i18n_js[] = 'monthStatsHelp';

        renderTemplate('monthStats', array(
                                     'module'               => $module,
                                     'header'               => $header,
                                     'blurb'                => $hBlurb,
                                     'stats'                => &$stats,
                                     'monthLabels'          => &$months,
                                     'month'                => $this->month,
                                     'year'                 => $year,
                                     'renderSelect'         => $this->renderSelect,
                                     'monthStatsBlurb'      => intl('monthStatsBlurb'),
                                     'monthStatsHelpLabel'  => intl('monthStatsHelpLabel')
                                     ));

    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function getCSS() {
        return array('month-stats.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Sean West <sean@linuxmagic.com>
     */
    public function getJS() {
        return array('month-stats.js', 'help-popup.js');
    }

    private function loadMonths() {
        return  array( 1  => intl('monthStatsJan'),
                       2  => intl('monthStatsFeb'),
                       3  => intl('monthStatsMar'),
                       4  => intl('monthStatsApr'),
                       5  => intl('monthStatsMay'),
                       6  => intl('monthStatsJun'),
                       7  => intl('monthStatsJul'),
                       8  => intl('monthStatsAug'),
                       9  => intl('monthStatsSep'),
                       10 => intl('monthStatsOct'),
                       11 => intl('monthStatsNov'),
                       12 => intl('monthStatsDec'));
    }
}
