<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2015 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: packageUpgradeModule.php 5666 2016-06-27 17:28:19Z shaun $
 */
require_once('lib.module.php');

class packageUpgradeModule extends Module {

    private $m_boolRunUpgrade;
    private $m_boolStatusUpdate;
    
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l,FALSE);
        $this->m_boolRunUpgrade = FALSE;
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // not much to see here... because this is a loop back ajax
        // module with 'real time' feedback, it is intended that all
        // actions and rendering take place hand in hand. However.. we
        // do set a 'mode' 
        switch ($action) {
        case 'run_upgrade':
            $this->m_boolRunUpgrade = TRUE;
            break;
        case 'status_update':
            $this->m_boolStatusUpdate = TRUE;
            break;
        default:
            break;
        }
    }

    //! method to render the full page module.
    public function renderModule() {
        renderTemplate('packageUpgrade', array(
                'pu_confirm' => intl('pu_confirm'),
                'pu_button_upgrade' => intl('pu_button_upgrade'),
                'cancel' => intl('cancel'),
            ));
    }

    //! method to render ajax request output
    public function renderOutput($modules) {
        if ($this->m_boolRunUpgrade) {
            // if the status file is already existing, then we abort. Someone else started an upgrade.
            if (file_exists(UPDATE_STAT_FILE)) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 200');
                return;
            }

            // run the upgrade.
            exec(SECURE_BIN . " " . PKG_UPDATER . PKG_UPDATE_ARGS . " >/dev/null 2>&1 &"); 

            // and done.  acknowledge the ajax call
            header($_SERVER['SERVER_PROTOCOL'] . ' 200');
        }
        if ($this->m_boolStatusUpdate) {
            if (file_exists(UPDATE_STAT_FILE)) {
                $status = trim(file_get_contents(UPDATE_STAT_FILE));
                header($_SERVER['SERVER_PROTOCOL'] . ' 200');
                if ($status == 'pu_install_complete') {
                    unlink(UPDATE_STAT_FILE);
                    // we send 'XXX:' as a prefix to let the Ajax caller know this is a full halt of status checks
                    print 'XXX:' . intl('pu_install_complete');
                    print "<div id=\"upgradeButtons\"><button class=\"msbutton\" onclick=\"completeUpgrade();\"><span>";
                    print intl('close');
                    print "</span></button></div>";
                } else if (substr($status, 0, 4) == 'ERR:') {
                    print "ERR:" . intl(substr($status,4));
                    print "<div id=\"upgradeButtons\">\n";
                    print "    <button class=\"msbutton\" onclick=\"startUpgrade();\">\n";
                    print "        <span>";
                    print intl('pu_retry');
                    print "        </span></button>\n";
                    print "    <button class=\"msbutton\" onclick=\"cancelUpgrade();\">\n";
                    print "        <span>";
                    print intl('cancel');
                    print "        </span>\n";
                    print "    </button></div>\n";
                    // in this case we unlink the file ourselves...
                    unlink(UPDATE_STAT_FILE);
                } else {
                    print intl($status);
                }
            }
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('package_upgrade.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('ajax.js', 'package_upgrade.js');
    }

    //! internal function to write a status update for upgrade.
    private function statusUpdate($msg) {
        if (  !isset($msg)
           || !is_string($msg)
           || !strlen(trim($msg)) ) {
            return;
        }
        $fp = fopen(UPDATE_STAT_FILE, "w");
        if (!$fp) {
            // uh oh... not much we can do...
            return;
        }
        fprintf($fp, "%s\n", trim($msg));
        fclose($fp);
    }
}
