<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: rateLimiterModule.php 11290 2020-03-27 16:26:44Z deiony $
 */
require_once("lib.module.php");

class rateLimiterModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array (
                                     'ipRateLimitList',
                                     'userRateLimitList',
                                  );
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ( $subModule && isset($modules[$subModule]) ) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module
    /*!
     * this is an override to allow mixed behavior between
     * 'container' and 'regular' module - for working with sub category
     * foldout widgets.
     */
    public function renderModule() {
        global $modules;
        $submodule = (isset($_POST['submodule'])
                      && strlen($_POST['submodule'])) ? $_POST['submodule'] : NULL;

        // Prints out the title for the page
        echo "<h2>" . intl('rateLimiter') . "</h2>";

        // Prints out the Master blurb for the page
        echo "<div class='masterBlurb'>";
        echo "<p>" . intl('rl_MasterBlurb') . "</p>";
        echo "</div>";

        $this->_i18n_js[] = 'exemptionsConfirmDelete';

        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }
            $activeAddon = "";
            // if TRUE, folds open by default
            $isActive = FALSE;
            $img = 'expand-icon.png';
            // we determine if this tab should be displayed expanded or not.
            // if no submodule was requested, then we don't expand anything
            if (!is_null($submodule)) {
                if ($m == $submodule) {
                    $isActive = TRUE;
                }
            }

            // Assemble the name of the session variable
            $collapseState = $m . "__collapse";
            
            // Validation to make certain we got a value
            if (isset($_SESSION[$collapseState]) && $_SESSION[$collapseState] != '') {
                $collapseString = $_SESSION[$collapseState];
            } else {
                $collapseString = 1;
            }
   
            // If 1 then we set it to be active
            if ($collapseString == 1) {
                $isActive = TRUE;
            }

            // Define the session variable here
            $_SESSION[$collapseState] = $collapseString;

            // Alters the style based if the tab is active
            if ($isActive) {
                $activeAddon = " collapsibleModuleActive";
                $img = 'collapse-icon.png';
                $firstExpanded = TRUE;
            }

            // Determines the value of the foldout title
            $foldoutTitle = "";
            if ($m == 'ipRateLimitList') {
                $foldoutTitle = intl('rl_ip_control_title');
            } else if ($m == 'userRateLimitList') {
                $foldoutTitle = intl('rl_auth_control_title');
            }

            echo "<div class=\"enforceGrey\">";
            echo "<div class=\"collapseHeader\" onclick=\"toggleSectionAndUpdateState(this, '{$m}','{$collapseString}', '" . IMAGEPATH . "');\">\n";
            echo "  <div class=\"collapseLeft\"></div>\n";
            echo "  <div class=\"collapseCenter\">\n";
            echo "    <span class=\"collapseTitle\">$foldoutTitle</span>\n";
            echo "    <span class=\"collapseActions\"><img src=\"" . IMAGEPATH . "{$img}\"/></span>\n";
            echo "  </div>\n";
            echo "  <div class=\"collapseRight\"></div>\n";
            echo "</div><!-- end collapseHeader-->\n";
            echo "<div id=\"{$m}\" class=\"collapsibleModule{$activeAddon}\">\n";

            $modules[$m]->renderModule();

            echo "</div><!-- end collapsibleModule -->\n";
            echo "</div><!-- end enforceGrey -->\n";
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array('ratelimiter.css', 'ratelimiter-list.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array('form-submit.js','limit-integer.js','rate-limiter.js', 'ajax.js', 'check-and-uncheck.js');
    }
}
