<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: settingsModule.php 10940 2020-01-14 00:44:10Z deiony $
 */
require_once("lib.modulecontainer.php");

class settingsModule extends ModuleContainer {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        $submodules = array('system_settings' => 'system',
                            'exemption'       => 'exemption',
                            'spam_settings'   => 'serverPolicies',
                            'ip_reputation'   => 'ipReputation',
                            'rate_limiter'    => 'rateLimiter',
                            );

        // these settings are only meant for the MS PRO series
        if (isProfessional()) {
            if (defined('SA_MANAGER')) {
                $submodules['spamAssassinSettings'] = 'spamAssassinSettings';
            }
            if (defined('USER_RULE_MANAGER')) {
                $submodules['user_settings'] = 'userSettings';
            }
        }

        parent::__construct($l, TRUE, $submodules);

        // set module to be migratable
        $this->migratable = TRUE;

        // Attempts to load each of the submodules
        global $modules;

        foreach ($this->submodules as $m) {
            if (array_key_exists($m, $modules)) {
                continue;
            }
            loadModule($m);
        }
    }

    //! method to return the title of the page to renderHeader
    /*!
     * @return  string  the value of the <title> of this page
     *
     * @author  Robert Bisewski <robertb@linuxmagic.com>
     */
    public function getPageTitle() {
        return intl('settings');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array_merge(array('form-submit.js', 'help-popup.js'),
                           $this->subModuleJS());
    }
}
