<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: statsModule.php 10940 2020-01-14 00:44:10Z deiony $
 */
require_once('lib.modulecontainer.php');

class statsModule extends ModuleContainer {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        $submodules = array('todayStats' => 'todayStats',
                            'monthStats' => 'monthStats',
                            'spamStats' => 'spamStats',
                            'inboundStats' => 'inboundStats',
                            'outboundStats' => 'outboundStats',
                            );

        // these settings are only meant for the MS PRO or PLUS series
        if (isProfessional()) {
            // only for the PRO series
            if (defined('SA_MANAGER')) {
                $submodules['spamAssassinStats'] = 'spamAssassinStats';
            } 
            // both PRO and PLUS series
            $submodules['countryStats'] = 'countryStats';
        }

        parent::__construct($l, TRUE, $submodules);

        foreach ($submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;

        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])
           && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to return the title of the page to renderHeader
    /*!
     * @return  string  the value of the <title> of this page
     * @author  Robert Bisewski <robertb@linuxmagic.com>
     */
    public function getPageTitle() {
        return intl('stats');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array_merge(array('form-submit.js', 'help-popup.js'), 
                           $this->subModuleJS());
    }
}
