<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: systemModule.php 9538 2019-03-14 23:30:28Z deiony $
 */
require_once('lib.module.php');

class systemModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->submodules = array(
            'protectionKey',
            'systemOptions',
            'migration',
        );

        // set module to be migratable
        $this->migratable = TRUE;

        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])
           && in_array($subModule, $this->submodules)) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;

        //Checks if all of the submodules are valid
        // XXX: interesting safety - but should it necessitate rendering
        //      a blank page if any single module is not valid?
        $allValid = true;
        foreach ($this->submodules as $idx => $m) {
            if (!is_array($modules) || !isset($modules[$m])) {
                $allValid = false;
                break;
            }
        }

        // If all of the submodules are valid, then the content is rendered
        if ($allValid) {
            foreach ($this->submodules as $m) {
                $modules[$m]->renderModule();
            }
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return $this->subModuleCSS();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return $this->subModuleJS();
    }
}
