<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2013 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: tzAutoModule.php 4839 2015-03-24 17:22:39Z shaun $
 */
require_once('lib.module.php');

class tzAutoModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        global $js;
        parent::__construct($l, FALSE);
        // set the timezone based on the javascript set cookie
        if (isset($_COOKIE['tzoffset'])) {
            $offset = intval($_COOKIE['tzoffset']) * -60;
            $tzname = timezone_name_from_abbr("", $offset, 0);
            if ($tzname === FALSE) {
                $abbrevs = timezone_abbreviations_list();
                if (is_array($abbrevs)) {
                     foreach ($abbrevs as $a) {
                        foreach ($a as $city) {
                            if ( $city['offset'] * -1 === $offset && $city['dst'] === FALSE) {
                                $tzname = $city['timezone_id'];
                                break;
                            }
                        }
                     }
                }
            }
            if (is_string($tzname)) {
                date_default_timezone_set($tzname);
            }
        } else {
            $js[] = 'tz.js';
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // this is just a place holder. The sole purpose of this
        // module is to include the javascript that in turn index.php
        // will use to set the time zone to the USER timezone.
    }

    //! method to render the full page module.
    public function renderModule() {
	// because this is an AJAX only module, we don't do anything here.
    }

   //! any ajax calls should be handle by this method
   /*
    * @param array $modules the global modules array
    * @param string $controller_file the filename of the controller
    *
    * NOTES: Any submodules that has ajax calls implement this method to handle the ajax calls
    */
    public function renderOutput($modules) {
        // not much to do here... it's an ajax call...with no feedback required.
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array();
    }
}

?>
