<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: ipReputationModule.php 4221 2014-11-04 23:28:57Z shaun $
 */
require_once("lib.module.php");

class userSettingsModule extends Module {
    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->migratable = TRUE;
        $this->submodules = array (
            'spamSettingsTemplate',
            'ipReputationTemplate',
        );
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;
        $subModule = isset($_POST['submodule']) ? $_POST['submodule'] : NULL;
        if ($subModule && isset($modules[$subModule])) {
            $modules[$subModule]->actionProcessor($action, $errors, $notices);
        }
    }


    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getCSS() {
        return array_merge($this->subModuleCSS(), array('user-settings.css'));
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     */
    public function getJS() {
        return array_merge(array('form-submit.js'), $this->subModuleJS());
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     *                 Kyle Macdonald <kylem@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;
        $firstExpanded = FALSE;
        $submodule = (isset($_POST['submodule']) && strlen($_POST['submodule'])) ? $_POST['submodule'] : NULL;

        // print out hte header and blurb
        // XXX: this whole module is a mess.. should be using templates instead of inline echo!
/*
        echo "<h2>" . intl('ip_reputation') . "</h2>";
        echo "<div class=\"masterBlurb\"><p>" . intl('ip_blurb') . "</p></div>";
*/

        foreach ($this->submodules as $m) {
            if (!isset($modules[$m])) {
                continue;
            }

            $modules[$m]->renderModule();

        }
    }
}
